@@ -2,7 +2,7 @@
 #define FONCTION_H
 
 #include <stdio.h>
-#include "wcs.h"
+#include <wcslib.h>
 #include "errors.h"
 #include <gsl/gsl_randist.h>
 
@@ -25,11 +25,6 @@
 
 #define CHECK_THIRD(A)   if(strlen(third) > A) { fprintf(stderr, "[ERROR] %s\n argument too long. Maximum argument size %d\n", third, A); exit(1); }
 
-#ifdef WCSLIB_WCS
-#define wcs2pix(A,B,C,D,E,F)  {double world[2],imgcrd[2],pixcrd[2]; world[0]=B;world[1]=C;wcss2p(A,1,2,world,D,E,imgcrd,pixcrd,F);*D=pixcrd[0];*E=pixcrd[1];}
-#define pix2wcs(A,B,C,D,E)  {double pixcrd[2],imgcrd[2],world[2]; int status; pixcrd[0]=B;pixcrd[1]=C;wcsp2s(A,1,2,pixcrd,imgcrd,D,E,world,&status);*D=world[0];*E=world[1];}
-#endif
-
 /*
  * structures in lenstool
  */
@@ -40,9 +35,9 @@
  * functions in lenstool
  */
 
-//#ifdef __cplusplus
-//"C" {
-//#endif
+#ifdef __cplusplus
+extern "C" {
+#endif
 
 complex acpx(complex c1, complex c2);
 complex acpxflt(complex c1, double f2);
@@ -74,6 +69,7 @@ double  comp_chi_osv(double *vect);
 double  comp_chi_osv(double *vect);
 void      comp_dchi_osv(double *vect, double *dvect);
 int   comp_desc(double A, double B);
+void cl_connectdots(int);
 void convertXY( double *x, double *y, int iref, double ra, double dec );
 void      copyright();
 void      copyTriplet(struct triplet *in, struct triplet *out);
@@ -99,6 +95,7 @@ complex dcpx(complex c1, complex c2);
 complex dcpxflt(complex c, double f);
 double  determinant(const struct point *A, const struct point *B, const struct point *C);
 double  d_gauss(double sig, int *idum);
+void    d_rbf(double **im, int nx, int ny, double scale);
 double  deltaVF(double zc);
 struct ellipse diag(double a, double b, double c);
 double  diff_mag(struct galaxie *arclet, struct point *guess);
@@ -119,7 +116,7 @@ double  d_random(int *idum);
 double  dratio(double zl, double zs);
 void dratio_gal(struct galaxie *arclet, double zl);
 double  dratioprime(double zl, double zs);
-void      d_rbfsig(double **im, int nx, int ny, double scale);
+void      d_rbf(double **im, int nx, int ny, double scale);
 double  d_rndschechter(int *idum);
 int       d_rndtype(int *idum);
 double  d_rndz(double zmax, int *idum);
@@ -158,6 +155,7 @@ double sersic_dpl(double r, double re, double n, double kappae);
 double sersic_kappa_eps(double r, double re, double n, double theta, double kappas, double eps);
 struct point sersic_gamma_eps(double r, double re, double n, double theta, double kappas, double eps);
 double err_invim(double **errim, int **imult);
+double e_sigma_ap(struct pot *ilens, double r);
 void   e_tau(long int n, struct galaxie gal[NAMAX]);
 void   e_testg(int i, int j, struct bitriplet *Tsol, int ni, struct point *P, double dlsds, double zs);
 int    e_test_P(struct bitriplet *Tsol, int ni, struct point *ps, struct point image[NIMAX], double dlsds, double zs, double err);
@@ -226,6 +224,7 @@ int     inverse(const struct point gsource[][NGGMAX], struct point *P, struct bi
 void    isoima(struct ellipse *es, struct ellipse *ampli, struct ellipse *ei);
 double  iter(double phi0, double coeur, double ct);
 void    keep_cl(double **image, struct pixlist *pl, int *npl);
+void    keep_cl_extended(struct pixlist *pl, int npl, struct pixlist *pl_ext, int *npl_ext); // mod bclement shapemodel_contour_ext
 int     lire(struct galaxie *liste, char *name);
 complex lncpx(complex c);
 void    local(int type, char localfile[]);
@@ -246,6 +245,7 @@ double  median(int n, double *array);
 double  mode(int n, double *array);
 void    multiscale_grid(long int *pilens);
 double  ncpx(complex c);
+void    newdo_itos(double **im, double dlsds, double zs, double **source, double **erreur, int **imult);
 double  nfw_dpl(double r, double rs, double kappas);
 double  nfw_gamma(double r, double rs, double kappas);
 struct point nfw_gamma_eps(double r, double rs, double theta, double kappas, double eps);
@@ -289,6 +289,7 @@ void    o_mag_m(int n, struct galaxie *arclet);
 double  o_min_loc(double y0);
 double  o_min_slope(double y0);
 void    o_pixel(double **zz, int nx, int ny, double scale, double xmin, double xmax, double ymin, double ymax, struct galaxie *source, double **dpl_x, double **dpl_y);
+void    o_pixel_contour(double **zz, int nx, int ny, double scale, double xmin, double xmax, double ymin, double ymax, struct galaxie *source, struct pixlist *pl, struct pixlist *pl_dpl, int npl); // mod bclement shapemodel_contour
 double  o_prep();
 void    o_prep_mult(int ntmult, struct galaxie *mult);
 void    o_print(FILE *OUT, double chi0);
@@ -400,6 +401,7 @@ void    read_lenstable();
 int     rescaleCube_1Atom(double *cube, int npar);
 int     reset_cosmoratio(); 
 double  rho_cri(double z);
+void    run_servermode();
 void    s_compmag(struct galaxie *gal, int *idum);
 void    scale_pot(struct g_pot *pot);
 complex scpx(complex c1, complex c2);
@@ -458,9 +460,11 @@ void    w_prop(int nprop, char propfile[]);
 void    wr_mass(char *name, double **map_xx, double **map_yy);
 void    wr_pot(char *name, double **map);
 void    w_sicat(struct galaxie ima[NFMAX][NIMAX], struct galaxie ss[NFMAX]);
+void wrf_from_header(char *filein, char *filename, int nx, int ny, double **tabpix);
 double  zero(double c1, double c2, double (*f)(double));
 double  zero_t(double c1, double c2, double (*f)(double));
 void    zonemult();
+void cl_connectdots(int verbose);
 
 //partie einasto
 
@@ -478,8 +482,36 @@ void read_table_einasto();
 void precompsource();
 void    testpixel(char *imname,char *sname,double *sflux,double **simu);
 
-//#ifdef __cplusplus
-//};
-//#endif
+#ifdef __cplusplus
+};
+#endif
+
+// Bspline Part
+//Benjamin Beauchesne - Bspline modification
+void knotx_from_ctrlpts(struct Bspline2D *Bspline, double **P);
+void knoty_from_ctrlpts(struct Bspline2D *Bspline, double **P);
+double Bspline_surface_point(const struct Bspline2D *Bspline, double u,double v);
+double Bspline_surface_point_der(struct Bspline2D *Bspline, double u,double v,int nx,int ny);
+double **controle_point_derive(int n, int p, int m, int q,double **P,double *U,double *V,int axes);
+int update_ctrl_pts(struct Bspline2D *Bspline,int n,double *coeff);
+int get_coeff_from_file(char *filename, struct Bspline2D *Bspline, int i);
+double **init_ctrl_pts(struct Bspline2D *Bspline);
+void knoty_init(struct Bspline2D *Bspline);
+void knotx_init(struct Bspline2D *Bspline);
+void init_derivate_Bspline(struct Bspline2D *Bspline);
+double Bspline_surface_point_red(const struct Bspline2D_red *Bspline, double u,double v);
+double **init_ctrl_pts_prec(struct Bspline2D *Bspline,double val);
+double **init_ctrl_pts_lim(struct Bspline2D *Bspline,struct Bspline2D *Bspline_coeff,int *block,double val,int a,double c,int d);
+void update_derivate_Bspline(struct Bspline2D *Bspline);
+struct point dpl_from_kappa(const struct Bspline2D *Bspline ,struct point *point,int n);
+struct matrix seconde_derivative_from_kappa(const struct Bspline2D *Bspline,struct point *point,int n);
+double poten_from_kappa(const struct Bspline2D *Bspline ,struct point *point,int n);
+
+double    **g_mass_tmp(int imass, int np, double zl, double zs);
+double    **g_shear_tmp(int ishear, int np, double z);
+double    **g_ampli_tmp(int iamp, int np, double z);
+double    **g_poten_tmp(int ipoten, int np, double z);
+double    ***g_dpl_tmp(int idpl, int np, double z);
+void inite_power_spectrum(struct Bspline2D *Bspline,double Fs,int N);
 
 #endif // if FONCTION_H
