@@ -1,9 +1,9 @@
 #ifndef STRUCTURE_H
 #define STRUCTURE_H
 
+#include <wcslib.h>
+#include <gsl/gsl_matrix.h>
 #include "dimension.h"
-#include<gsl/gsl_matrix.h>
-#include "wcs.h"
 
 // Parameter constants
 #define CX       0
@@ -33,19 +33,34 @@
 #define STHETA  24
 #define SINDEX  25
 #define SFLUX   26
-#define VFCX    27
-#define VFCY    28
-#define VFVT    29
-#define VFRT    30
-#define VFI     31
-#define VFTHETA 32
-#define VFLCENT 33
-#define VFSIGMA 34
+#define SA2     27
+#define SEPS2   28
+#define STHETA2 29
+#define SINDEX2 30
+#define SFLUX2  31
+#define VFCX    32
+#define VFCY    33
+#define VFVT    34
+#define VFRT    35
+#define VFI     36
+#define VFTHETA 37
+#define VFLCENT 38
+#define VFSIGMA 39
+//Benjamin Beauchesne - Bspline modification
+#define CX_PERT 40 //must be the first for perturbative element
+#define SIZE_LATT 41
+#define CY_PERT 42 // must be the last
 
 /*
 * structure definition
 */
 
+#ifndef __cplusplus
+#define false 0
+#define true 1
+#define bool int
+#endif
+
 /*****************************************************************/
 /*                               */
 /*      Definition de type               */
@@ -235,11 +250,13 @@ struct g_mode
 	int     iclean;
 	double  zclean;
         int     flux; //conserve surface brightness ? 1=True 0=False
+        int     output; //writes models and residuals from shapemodel
 	/* center of multiple images file */
 	char    centerfile[FILENAME_SIZE];
     /* bayesys restart flag */
     int     restart;
     int     restart_seed;
+    char    servermode_url[200];  //URL for server mode
 };
 
 /* potfile parameters */
@@ -303,12 +320,9 @@ struct g_pixel
 	double  ymax;
 	int     nx;
 	int     ny;
-        double  meanFlux;
-#ifdef WCSLIB_WCS
-	struct  wcsprm *wcsinfo;
-#else 
-	struct  WorldCoor *wcsinfo;
-#endif
+    double  meanFlux;
+    struct  wcsprm *wcsinfo;
+    double  **array;
 };
 
 struct g_cube
@@ -327,13 +341,10 @@ struct g_cube
 	int     nx;
 	int     ny;
 	int     nz;
-        int header;
-        double  meanFlux;
-#ifdef WCSLIB_WCS
-	struct  wcsprm *wcsinfo;
-#else
-	struct  WorldCoor *wcsinfo;
-#endif
+    int     header;
+    double  meanFlux;
+    struct  wcsprm *wcsinfo;
+    double  ***array;
 };
 
 struct g_image
@@ -391,6 +402,13 @@ struct g_image
 
 struct g_source
 {
+        // benjamin Beauchesne teste true_source
+        struct point multi_s[NFMAX];
+        char    multfile[FILENAME_SIZE];
+        int     n_mult;
+	int     multmode;
+        int     mult[NFMAX];
+
 	int     grid;
 	int     rand;
 	long int  ns;
@@ -496,6 +514,74 @@ struct g_cosmo
     double  h;
 };
 
+
+/*****************************************************************/
+/*                               */
+/*       Definition des structures des surface de Bspline       */
+/*                               */
+/*****************************************************************/
+
+//Benjamin Beauchesne - Bspline modification
+struct Bspline3D
+{
+	int degx; // degree of Bspline surface in x-direction
+	int degy; // degree of Bspline surface in y-direction
+	int n; // n*n Number of controle points which are variable
+	double **Pz; // array of control points z-coordinate
+	double **Py; // array of control points y-coordinate
+	double **Px; // array of control points x-coordinate
+	double *kx; // knot vector in x-direction
+	double *ky; // knot vector in x-direction
+        double x_c; // coordinate of the center of perturbation
+        double y_c;
+        double size_sq; // Size of the square of the lattice
+};
+
+struct Bspline2D_red
+{
+	int degx; // degree of Bspline surface in x-direction
+	int degy; // degree of Bspline surface in y-direction
+        int nx; // Number of controle point in x-direction
+        int ny; // Number of controle point in x-direction
+	double **Pz; // array of control points z-coordinate
+        double *kx; // knot vector in x-direction
+	double *ky; // knot vector in x-direction
+
+
+};
+
+
+
+struct Bspline2D
+{
+	int degx; // degree of Bspline surface in x-direction
+	int degy; // degree of Bspline surface in y-direction
+	int n; // n*n Number of controle points which are variable
+        int nx; // Number of controle point in x-direction
+        int ny; // Number of controle point in x-direction
+	double **Pz; // array of control points z-coordinate
+        struct Bspline2D_red der10; //Bspline x-derivate 
+        struct Bspline2D_red der01; //Bspline y-derivate
+        struct Bspline2D_red der20; //Bspline x-derivate two times
+        struct Bspline2D_red der11; //Bspline x-derivate and y-derivate
+        struct Bspline2D_red der02; //Bspline y-derivate two times
+
+        
+        double *kx; // knot vector in x-direction
+	double *ky; // knot vector in x-direction
+        double x_c; // coordinate of the center of perturbation
+        double y_c;
+        double size_sq; // Size of the square of the lattice
+        double lim; // use to stored the value of the limit
+        char   coeff_file[FILENAME_SIZE]; // name of coefficient files
+};
+// If Pz is modified derivate must be change to [0,0] or Pzdev==NULL. If not
+//the derivate could be wrong or based on the ancient value of Pz
+
+
+
+
+
 /*****************************************************************/
 /*                               */
 /*              Definition des types             */
@@ -592,10 +678,12 @@ struct galaxie
 	struct point    C;
 	struct point    Grad;   // total deflection with all clumps
 	struct ellipse  E;
+	struct ellipse  E2;
 	char    c;
 	int     type;
 	double  magabs;
 	double  mag;
+	double  mag2;
 	double  flux;
 	double  mu;
 	double  I0;
@@ -605,6 +693,7 @@ struct galaxie
 	double  dr;           // ratio dl0s/dos
 	double  q;
 	double  eps;
+	double  eps2;
 	double  tau;
 	double  dis;
 	double  A;
@@ -752,19 +841,14 @@ struct pot
 //      double  wX;
 	double  costheta;
 	double  sintheta;
-#ifdef WCSLIB_WCS
-	struct wcsprm *wcsinfo;
-#else
-	struct WorldCoor *wcsinfo;
-#endif
-	double **dplxmap;
-	double **dplymap;
-	double **kappamap;
-	double **gamma1map;
-	double **gamma2map;
-	double **potenmap;
-    int map_nxpix;
-    int map_nypix;
+	struct g_pixel *dplxmap;
+	struct g_pixel *dplymap;
+	struct g_pixel *kappamap;
+	struct g_pixel *gamma1map;
+	struct g_pixel *gamma2map;
+	struct g_pixel *potenmap;
+        
+    struct Bspline2D Bspline; //Benjamin Beauchesne - Bspline modification
 };
 
 struct ipot
@@ -819,6 +903,8 @@ struct pixlist
 {
 	double  i;
 	double  j;
+        int ii; // mod bclement shapemodel_contour
+        int jj; // mod bclement shapemodel_contour
 	double flux;
 };
 
@@ -844,4 +930,9 @@ struct vfield
 	int profile;
 };
 
+
+
+
+
+
 #endif // if STRUCTURE_H
