@@ -28,5 +28,10 @@ typedef struct          // INDIVIDUAL PARAMETERS
   .....
 } UserObjectStr;
 */
+#ifdef __cplusplus
+extern "C" FILE* open_mcmc_files(double cool, char *suffix);
+extern "C" void write_bayes_line(FILE *bayes, double o_lhood_rez, double last_col, long int samp, UserCommonStr* UserCommon);
+#endif
+
 
 #endif
