@@ -50,5 +50,5 @@ double three_eighths(double a, double b, int N,double (*func)(double))
 }
 
 
-#undef   FUNC(x) ((*func)(x))
+#undef   FUNC
 
