@@ -52,5 +52,5 @@ double three_eighths2(double a, double b,double c,double d, int N,double (*func)
 }
 
 
-#undef   MYFUNC(x,y,z) ((*func)(x,y,z))
+#undef   MYFUNC
 
