@@ -817,3 +817,85 @@ int wrf_cube_fits_abs(char *filename, double ***cube, int nx,int ny, int nz,
     
     return 0;
 }
+
+/* write output image based on known header */
+void wrf_from_header(char *filein, char *filename, int nx, int ny, double **tabpix)
+{
+    fitsfile *infptr;
+    fitsfile *outfptr;
+    int status;
+    int nkeys;
+    int fpixel;
+    int nelements;
+    float *p_ima;
+    int ii,jj,k;
+    int bitpix=FLOAT_IMG;
+    long naxes[2] = {nx,ny};
+    long naxesin[2] = {0,0};
+    long npixels;
+    int hdutype;
+    int nfound;
+
+    status = 0;         /* initialize status before calling fitsio routines */
+
+    if( fits_open_file(&infptr, filein, READONLY, &status) )
+            printerror( status );
+
+
+    fpixel = 1;                             /* first pixel in the image    */
+    ii=0;
+    npixels=0;
+
+
+    /* move to first extension of input file with data */
+    do
+    {
+		ii++;
+                if (fits_movabs_hdu(infptr,ii, &hdutype, &status))
+                       printerror( status );
+		if( fits_read_keys_lng(infptr, "NAXIS", 1, 2, naxesin, &nfound, &status) )
+			printerror( status );
+		npixels = naxesin[0] * naxesin[1];          /* number of pixels in the input image */
+    }
+    while(npixels == 0);
+
+    fpixel = 1;
+    nelements = nx*ny;
+    p_ima=(float *)malloc((unsigned) (nelements)*sizeof(float));
+    k=0;
+    for (jj=0;jj<ny;jj++)
+        for (ii=0;ii<nx;ii++)
+        {
+                p_ima[k]=(float)tabpix[jj][ii];
+                k++;
+        }
+    remove(filename);               /* Delete old file if it already exists */
+
+
+    status = CREATE_DISK_FILE;         /* initialize status before calling fitsio routines */
+    if (fits_create_file(&outfptr, filename, &status)) /* create new FITS file */
+         printerror( status );           /* call printerror if error occurs */
+
+    if ( fits_create_img(outfptr, FLOAT_IMG, 2, naxes, &status) )
+        printerror( status );
+
+    if ( fits_copy_header(infptr, outfptr, &status) )
+        printerror( status );
+
+    if ( fits_write_img(outfptr, TFLOAT, fpixel, nelements, p_ima, &status) )
+        printerror( status );
+
+    if (fits_movabs_hdu(outfptr,1, &hdutype, &status))
+        printerror( status );
+
+    if (fits_delete_hdu(outfptr, &hdutype, &status))
+        printerror( status );
+
+    if ( fits_close_file(infptr, &status) )                /* close the file */
+         printerror( status );
+
+    if ( fits_close_file(outfptr, &status) )
+            printerror(status);
+    free(p_ima);
+}
+
