@@ -31,7 +31,7 @@ void    classer(struct galaxie image[NFMAX][NIMAX],
     *ni = k;
 
     // note tous les numeros des sources donnant des arcs geants
-    for (i = 0; i < *ni && cimage[i].tau > L.dlarge ; i++);
+    for (i = 0; i < *ni && cimage[i].tau > L.dlarge ; i++)
         strcpy(ngiant[i], cimage[i].n); 
 
     *ncistart = i;
