@@ -39,9 +39,15 @@ void    criticnew(int verbose)
     extern struct   g_cline CL;
 
     if ( !strcmp(CL.algorithm, "SNAKE") )
+    {
         snake(verbose);
+	w_critic();
+    }
     else
+    {
         marchingSquares(verbose);
+    }
+    
 
 }
 
@@ -201,6 +207,8 @@ static  void cutSquare(
     double x, double y, double width, double height,
     unsigned char parentType, int squareNb, double *parentAmpli )
 {
+//#define DEBUG 1
+
 #ifdef DEBUG
     extern struct g_mode M;
     FILE * dbg;
@@ -563,6 +571,109 @@ static void marchingSquares(int verbose)
     }
 }
 
+void cl_connectdots(int verbose)
+{
+    extern struct g_cline       CL;
+    extern  struct  biline  radial[], tangent[];
+    extern  int nrline, ntline;
+//#ifdef DEBUG
+        fprintf( stderr, "Connecting the marchingSquares segments....\n" );
+//#endif
+
+    float currentdist,distmin,disttest;
+    struct biline zpos,currentpos;
+    int pos,minpos,iline;
+    int i;
+    
+    iline=1;
+
+    currentpos=tangent[1]; // start with the first two points
+
+    for(i=2;i<ntline;i++)
+    {
+	 if(tangent[i].i==1)  // start a new line
+	 {
+		 iline++;
+		 currentpos=tangent[i];
+                 tangent[i].i=iline;
+		 continue;
+	 }
+	 pos=i;
+	 distmin=-1;
+	 minpos=i;
+	 while((tangent[pos].i!=1)&&(pos<ntline))
+         {
+	    currentdist=pow(currentpos.I.x-tangent[pos].I.x,2)+pow(currentpos.I.y-tangent[pos].I.y,2);
+	    if((distmin<0)||(currentdist<distmin))
+	    {  
+		    minpos=pos;
+		    distmin=currentdist;
+	    }
+	    pos++;
+         }
+
+	 if(distmin>pow(CL.limitHigh,2.0))
+         {
+              iline++;
+	      currentpos=tangent[i];
+              tangent[i].i=iline;
+	      continue;
+	 }
+
+	 //swap position i and minpos
+	 zpos=tangent[i];
+	 tangent[i]=tangent[minpos];
+	 tangent[minpos]=zpos;
+	 currentpos=tangent[i];
+         tangent[i].i=iline;
+    }
+
+    iline=1;
+
+    currentpos=radial[1]; // start with the first two points
+
+    for(i=2;i<nrline;i++)
+    {
+	 if(radial[i].i==1)  // start a new line
+	 {
+		 iline++;
+		 currentpos=radial[i];
+                 radial[i].i=iline;
+		 continue;
+	 }
+	 pos=i;
+	 distmin=-1;
+	 minpos=i;
+	 while((radial[pos].i!=1)&&(pos<nrline))
+         {
+	    currentdist=pow(currentpos.I.x-radial[pos].I.x,2.0)+pow(currentpos.I.y-radial[pos].I.y,2.0);
+	    if((distmin<0)||(currentdist<distmin))
+	    {  
+		    minpos=pos;
+		    distmin=currentdist;
+	    }
+	    pos++;
+         }
+
+	 //disttest=pow(currentpos.I.x-radial[startpos].I.x,2)+pow(currentpos.I.y-radial[startpos].I.y,2);
+	 //if((disttest<distmin)&&(disttest>0))
+	 if(distmin>pow(CL.limitHigh,2.0))
+         {
+              iline++;
+	      currentpos=radial[i];
+              radial[i].i=iline;
+	      continue;
+	 }
+	 
+	 //swap position i and minpos
+	 zpos=radial[i];
+	 radial[i]=radial[minpos];
+	 radial[minpos]=zpos;
+	 currentpos=radial[i];
+         radial[i].i=iline;
+    }
+}
+
 
 /* Find the caustic/critical lines with the snake algorithm.
  *
