@@ -8,6 +8,7 @@
 #include<string.h>
 
 
+
 static double hypo(double x, double y, long int i);
 static struct point e_grad_np(const struct galaxie *image, double *np_b0);
 
@@ -48,7 +49,7 @@ struct point e_grad_pot(const struct point *pi, long int i)
     g.x = g.y = 0.;
 
     // shortcut for N atoms
-    if (ilens->b0 == 0. && ilens->type != 14 && ilens->type !=17)
+    if (ilens->b0 == 0. && ilens->type != 14 && ilens->type !=17 && ilens->type !=814)
         return g;
    
     /*positionning at the potential center*/
@@ -301,7 +302,89 @@ struct point e_grad_pot(const struct point *pi, long int i)
                 Grad.x = 0.;
                 Grad.y = 0.;
             }
-            break; 
+            break;
+        //Benjamin Beauchesne - Bspline modification
+        case(814): // PIEMD with a Bspline perturbation
+            /*
+            // Compute initial PIEMD
+            if ( ilens->epot > 2E-4 )
+            {
+                t05 = ilens->b0 * ilens->rcut / (ilens->rcut - ilens->rc);
+                //zis = ci05(Q.x, Q.y, ilens->epot, ilens->rc);
+                //zis_cut = ci05(Q.x, Q.y, ilens->epot, ilens->rcut);
+                //g.x = t05 * (zis.re - zis_cut.re);
+                //g.y = t05 * (zis.im - zis_cut.im);
+		zis = ci05f(Q.x, Q.y, ilens->epot, ilens->rc, ilens->rcut);
+		g.x = t05 * zis.re;
+		g.y = t05 * zis.im;
+            }
+            else if ((u = Q.x * Q.x + Q.y * Q.y) > 0. )
+            {
+                // Circular dPIE Elliasdottir 2007 Eq A23 slighly modified for t05
+                X = ilens->rc;
+                Y = ilens->rcut;
+                t05  = sqrt(u + X * X) - X - sqrt(u + Y * Y) + Y;  // Faster and equiv to Elliasdottir (see Golse PhD)
+                t05 *= ilens->b0 * Y / (Y - X) / u; // 1/u because t05/sqrt(u) and normalised Q/sqrt(u)
+                g.x = t05 * Q.x;
+                g.y = t05 * Q.y;
+            }
+            else
+            {
+                g.x = 0.;
+                g.y = 0.;
+            }
+            
+            */
+            // Compute Bspline perturbation
+            //double tmp1, tmp2;
+            g.x=Bspline_surface_point_red(&ilens->Bspline.der10,Q.x,Q.y);
+            g.y=Bspline_surface_point_red(&ilens->Bspline.der01,Q.x,Q.y);
+            //printf("%f %f %f %f %f %f %f \n",ilens->b0,ilens->rcut,ilens->rc, ilens->epot, ilens->emass,ilens->Bspline.x_c,ilens->Bspline.y_c); 
+            break;
+
+        case(815): // PIEMD with a Bspline perturbation
+
+            // Compute initial PIEMD
+            if ( ilens->epot > 2E-4 )
+            {
+                t05 = ilens->b0 * ilens->rcut / (ilens->rcut - ilens->rc);
+                //zis = ci05(Q.x, Q.y, ilens->epot, ilens->rc);
+                //zis_cut = ci05(Q.x, Q.y, ilens->epot, ilens->rcut);
+                //g.x = t05 * (zis.re - zis_cut.re);
+                //g.y = t05 * (zis.im - zis_cut.im);
+		zis = ci05f(Q.x, Q.y, ilens->epot, ilens->rc, ilens->rcut);
+		g.x = t05 * zis.re;
+		g.y = t05 * zis.im;
+            }
+            else if ((u = Q.x * Q.x + Q.y * Q.y) > 0. )
+            {
+                // Circular dPIE Elliasdottir 2007 Eq A23 slighly modified for t05
+                X = ilens->rc;
+                Y = ilens->rcut;
+                t05  = sqrt(u + X * X) - X - sqrt(u + Y * Y) + Y;  // Faster and equiv to Elliasdottir (see Golse PhD)
+                t05 *= ilens->b0 * Y / (Y - X) / u; // 1/u because t05/sqrt(u) and normalised Q/sqrt(u)
+                g.x = t05 * Q.x;
+                g.y = t05 * Q.y;
+            }
+            else
+            {
+                g.x = 0.;
+                g.y = 0.;
+            }
+
+
+            // Compute Bspline perturbation
+            //double tmp1, tmp2;
+            /*
+            g.x+=Bspline_surface_point_red(&ilens->Bspline.der10,Q.x,Q.y);
+            g.y+=Bspline_surface_point_red(&ilens->Bspline.der01,Q.x,Q.y);
+            */ 
+            struct point point=dpl_from_kappa(&ilens->Bspline ,&Q,128);
+            g.x+=point.x;
+            g.y+=point.y;
+            //printf("%f %f \n",Q.x,Q.y);
+            break;
+
         case(82): /* PIEMD kovner with shallow central slope I0.5c+I0.5cut*/
 
             t05 = ilens->b0 * (ilens->rcut + ilens->rc) / ilens->rcut;
@@ -503,21 +586,26 @@ struct point e_grad_pot(const struct point *pi, long int i)
             g.x = dpl_rad * (1. - ilens->emass) * Q.x / u;        // corrections to elliptical potential
             g.y = dpl_rad * (1. + ilens->emass) * Q.y / u;
             break;
-	case(17): // external displacement maps
-	        xw = Q.x / (-3600.) / cos(M.ref_dec * DTR) + M.ref_ra;
-            yw = Q.y / 3600. + M.ref_dec;
-            wcs2pix(ilens->wcsinfo, xw, yw, &xpixel, &ypixel, &offscl);
-            if (offscl == 0)
+    
+        case(17): ; // external displacement maps
+
+            double world[2], imgcrd[2], pixcrd[2];
+            double phi[1], theta[1];
+            int stat[1], status;
+            world[0] = Q.x / (-3600.) / cos(M.ref_dec * DTR) + M.ref_ra;
+            world[1] = Q.y / 3600. + M.ref_dec;
+            if (wcss2p(ilens->kappamap->wcsinfo, 1, 2, world, phi, theta, imgcrd, pixcrd, stat)==0)
             {
-	            g.x=bilinear(ilens->dplxmap,xpixel,ypixel,ilens->map_nxpix,ilens->map_nypix);
-	            g.y=bilinear(ilens->dplymap,xpixel,ypixel,ilens->map_nxpix,ilens->map_nypix);
+                g.x=bilinear(ilens->dplxmap->array,pixcrd[0],pixcrd[1],ilens->dplxmap->nx,ilens->dplxmap->ny);
+                g.y=bilinear(ilens->dplymap->array,pixcrd[0],pixcrd[1],ilens->dplymap->nx,ilens->dplymap->ny);
             }
-		    else
-		    {
-		        g.x=0.0;
+            else
+            {
+                g.x=0.0;
                 g.y=0.0;
-		    }
-	    break;
+            }
+            break;
+
         default:
             fprintf(stderr, "ERROR: profil type of clump %s unknown : %d\n",
                     ilens->n, ilens->type);
