@@ -6,6 +6,8 @@
 #include<constant.h>
 #include<dimension.h>
 #include<structure.h>
+
+
 //#include "omp.h"
 
 #define cube(A) A*A*A
@@ -310,6 +312,7 @@ struct matrix e_grad2_pot_ptr(const struct point *pi, const struct pot *ilens)
     struct matrix  grad2, g2, g05c, g05cut, g10c, g10cut, g15c, g15cut; //,g21,g20
     struct matrix  gsiemd;
     struct point   Qe, dlenspotent;  // elliptical radius NFW/Sersic
+    double saved;
 
     double  kap, tell;
     struct point gamma;
@@ -318,14 +321,11 @@ struct matrix e_grad2_pot_ptr(const struct point *pi, const struct pot *ilens)
     complex zis_cut, zis ;
     double newx,newx_x,newx_y,newx_xx,newx_yy,newx_xy,newy,newy_x,newy_y,newy_xx,newy_yy,newy_xy;
 
-    double xw,yw,xpixel,ypixel;
-    int offscl;
-
     g2.a = g2.b = g2.c = g2.d = 0.;
 
     dlenspotent.x =dlenspotent.y =0.;
     // shortcut for N atoms
-    if (ilens->b0 == 0. && ilens->type != 14 && ilens->type !=17)
+    if (ilens->b0 == 0. && ilens->type != 14 && ilens->type !=17 && ilens->type !=814)
         return g2;
 
     R.x = pi->x - ilens->C.x;
@@ -656,6 +656,108 @@ if ( ilens->epot > 2E-25 )
                 g2.b = g2.d = 0.;
             }
             break;
+        //Benjamin Beauchesne - Bspline modification
+        case(814): // PIEMD with a Bspline perturbation
+            /*
+            // Compute initial PIEMD
+            if ( ilens->epot > 2E-4 )
+            {
+                t05 = ilens->rcut / (ilens->rcut - ilens->rc);
+                mdci05(Q.x, Q.y, ilens->epot, ilens->rc, ilens->b0, &g05c);
+                mdci05(Q.x, Q.y, ilens->epot, ilens->rcut, ilens->b0, &g05cut);
+                g2.a = t05 * (g05c.a - g05cut.a);
+                g2.b = t05 * (g05c.b - g05cut.b);
+                g2.c = t05 * (g05c.c - g05cut.c);
+                g2.d = t05 * (g05c.d - g05cut.d);
+            }
+            else if ( (RR = Q.x * Q.x + Q.y * Q.y) > 0. )
+            {
+                // Circular dPIE Elliasdottir 2007 Eq A23 slighly modified for t05
+                X = ilens->rc;
+                Y = ilens->rcut;
+                t05 = ilens->b0 * Y / (Y - X); // 1/u because t05/sqrt(u) and normalised Q/sqrt(u)
+                z  = sqrt(RR + X * X) - X - sqrt(RR + Y * Y) + Y;  // R*dphi/dR
+                X = RR / X;
+                Y = RR / Y;
+                p  = (1. - 1. / sqrt(1. + X / ilens->rc)) / X - (1. - 1. / sqrt(1. + Y / ilens->rcut)) / Y;  // d2phi/dR2
+                X = Q.x * Q.x / RR;
+                Y = Q.y * Q.y / RR;
+                g2.a = t05 * (p * X + z * Y / RR);
+                g2.c = t05 * (p * Y + z * X / RR);
+                X = Q.x * Q.y / RR;
+                g2.b = g2.d = t05 * (p * X - z * X / RR);
+            }
+            else
+            {
+                g2.a = g2.c = ilens->b0 / ilens->rc / 2.;
+                g2.b = g2.d = 0.;
+            }
+            */
+            // Compute Bspline perturbation
+            //double tmp1,tmp2;
+            
+            saved=Bspline_surface_point_red(&ilens->Bspline.der11,Q.x,Q.y);
+            g2.a=Bspline_surface_point_red(&ilens->Bspline.der20,Q.x,Q.y);
+            g2.b =saved;
+            g2.d =saved;
+            g2.c=Bspline_surface_point_red(&ilens->Bspline.der02,Q.x,Q.y);
+            
+            break;
+
+        case(815): // PIEMD with a Bspline perturbation
+
+            // Compute initial PIEMD
+            if ( ilens->epot > 2E-4 )
+            {
+                t05 = ilens->rcut / (ilens->rcut - ilens->rc);
+                mdci05(Q.x, Q.y, ilens->epot, ilens->rc, ilens->b0, &g05c);
+                mdci05(Q.x, Q.y, ilens->epot, ilens->rcut, ilens->b0, &g05cut);
+                g2.a = t05 * (g05c.a - g05cut.a);
+                g2.b = t05 * (g05c.b - g05cut.b);
+                g2.c = t05 * (g05c.c - g05cut.c);
+                g2.d = t05 * (g05c.d - g05cut.d);
+            }
+            else if ( (RR = Q.x * Q.x + Q.y * Q.y) > 0. )
+            {
+                // Circular dPIE Elliasdottir 2007 Eq A23 slighly modified for t05
+                X = ilens->rc;
+                Y = ilens->rcut;
+                t05 = ilens->b0 * Y / (Y - X); // 1/u because t05/sqrt(u) and normalised Q/sqrt(u)
+                z  = sqrt(RR + X * X) - X - sqrt(RR + Y * Y) + Y;  // R*dphi/dR
+                X = RR / X;
+                Y = RR / Y;
+                p  = (1. - 1. / sqrt(1. + X / ilens->rc)) / X - (1. - 1. / sqrt(1. + Y / ilens->rcut)) / Y;  // d2phi/dR2
+                X = Q.x * Q.x / RR;
+                Y = Q.y * Q.y / RR;
+                g2.a = t05 * (p * X + z * Y / RR);
+                g2.c = t05 * (p * Y + z * X / RR);
+                X = Q.x * Q.y / RR;
+                g2.b = g2.d = t05 * (p * X - z * X / RR);
+            }
+            else
+            {
+                g2.a = g2.c = ilens->b0 / ilens->rc / 2.;
+                g2.b = g2.d = 0.;
+            }
+            // Compute Bspline perturbation
+            //double tmp1,tmp2;
+            /*
+            saved=Bspline_surface_point_red(&ilens->Bspline.der11,Q.x,Q.y);
+            g2.a+=Bspline_surface_point_red(&ilens->Bspline.der20,Q.x,Q.y);
+            g2.b +=saved;
+            g2.d +=saved;
+            g2.c+=Bspline_surface_point_red(&ilens->Bspline.der02,Q.x,Q.y);
+            */
+            struct matrix matB=seconde_derivative_from_kappa(&ilens->Bspline,&Q,128);
+            g2.a+=matB.a;
+            g2.b +=matB.b;
+            g2.d +=matB.d;
+            g2.c+=matB.c;
+            //printf("%f %f %f %f \n",matB.a,matB.d,matB.b,matB.c); 
+
+            break;
+
+
         case(82): /* PIEMD kovner with a shallower central slope*/
             t05 = (ilens->rcut + ilens->rc) / ilens->rcut;
             mdci05(Q.x, Q.y, ilens->epot, ilens->rc, ilens->b0, &g05c);
@@ -893,27 +995,33 @@ if ( ilens->epot > 2E-25 )
             g2.b = g2.d = gamma.y;
             g2.c = kap + gamma.x;
             break;
-	case(17):
-	    xw = Q.x / (-3600.) / cos(M.ref_dec * DTR) + M.ref_ra;
-            yw = Q.y / 3600. + M.ref_dec;
-            wcs2pix(ilens->wcsinfo, xw, yw, &xpixel, &ypixel, &offscl);
-            if (offscl == 0)
-            {
-    		   kap=bilinear(ilens->kappamap,xpixel,ypixel,ilens->map_nxpix,ilens->map_nypix);
-	           gamma.x=bilinear(ilens->gamma1map,xpixel,ypixel,ilens->map_nxpix,ilens->map_nypix);
-	           gamma.y=bilinear(ilens->gamma2map,xpixel,ypixel,ilens->map_nxpix,ilens->map_nypix);
 
-               g2.a = kap - gamma.x;
-               g2.b = g2.d = gamma.y;
-               g2.c = kap + gamma.x;
-            }
-	    else
+        case(17): ;  // external displacement maps
+            // semi-colon to prevent error: a label cannot be followed by a declaration in C99
+
+            double world[2], imgcrd[2], pixcrd[2];
+            double phi[1], theta[1];
+            int stat[1], status;
+            world[0] = Q.x / (-3600.) / cos(M.ref_dec * DTR) + M.ref_ra;
+            world[1] = Q.y / 3600. + M.ref_dec;
+            if (wcss2p(ilens->kappamap->wcsinfo, 1, 2, world, phi, theta, imgcrd, pixcrd, stat)==0)
             {
-                   g2.a = 0.0;
-                   g2.b = g2.d = 0.0;
-                   g2.c = 0.0;
-	    }
-	    break;
+                kap=bilinear(ilens->kappamap->array,pixcrd[0],pixcrd[1],ilens->kappamap->nx,ilens->kappamap->ny);
+                gamma.x=bilinear(ilens->gamma1map->array,pixcrd[0],pixcrd[1],ilens->gamma1map->nx,ilens->gamma1map->ny);
+                gamma.y=bilinear(ilens->gamma2map->array,pixcrd[0],pixcrd[1],ilens->gamma2map->nx,ilens->gamma2map->ny);
+
+                g2.a = kap + gamma.x;
+                g2.b = g2.d = gamma.y;
+                g2.c = kap - gamma.x;
+            }
+            else
+            {
+                g2.a = 0.0;
+                g2.b = g2.d = 0.0;
+                g2.c = 0.0;
+            }
+            break;
+
         default:
             fprintf(stderr, "ERROR: profil type of clump %s unknown: %d \n",
                     ilens->n, ilens->type);
