@@ -42,6 +42,9 @@ double  interpol(double xx, const double *fxx, const double *fyy, const double *
 double bilinear(double **map, double x, double y, int xmax, int ymax)
 {
     int x1,y1;
+//Difference of wcs2pix reference (starts at 1,1) and map reference (starts at pixel 0,0)
+    x=x-1;
+    y=y-1;
     x1=(int)(x-0.5);
     y1=(int)(y-0.5);
     if(x1<0) x1=0;
