@@ -91,6 +91,15 @@ int o_chi_lhood0(double *chi_ext, double *lhood0_ext, double *np_b0)
 //    extern struct pixlist plo[];
     extern double **imo, **wo, **ero, **soo, lhood_wo;
 
+    // mod bclement shapemodel_contour
+    extern struct pixlist *imo_pl;
+    extern int imo_npl;
+    // mod bclement shapemodel_contour_ext
+    extern struct pixlist *imo_pl_ext;
+    extern struct pixlist *imo_pl_dpl_ext;
+    extern int imo_npl_ext;
+    int n;
+
     /* local variables */
     struct point A, B, D;
     struct ellipse ampli;
@@ -98,6 +107,7 @@ int o_chi_lhood0(double *chi_ext, double *lhood0_ext, double *np_b0)
     double chi, lhood0;
     double x, qp, tp;
     int i, j;
+    int sizex,sizey;
 
 #ifdef CHIRES
     OUT = fopen(filename, "w");
@@ -183,10 +193,16 @@ int o_chi_lhood0(double *chi_ext, double *lhood0_ext, double *np_b0)
 		fprintf(stderr, "Error | imFrame.pixelx(%f) != imFrame.pixely(%f) but we assume that they are equal\n", imFrame.pixelx, imFrame.pixely);
 		exit(EXIT_FAILURE);
 	     }
-	    
-	    o_pixel(ero, imFrame.nx, imFrame.ny, imFrame.pixelx, imFrame.xmin, imFrame.xmax, imFrame.ymin, imFrame.ymax, source, map_axx, map_ayy);
-	   	   
-        //wrf_fits_abs("simu.fits", ero, imFrame.nx, imFrame.ny, imFrame.xmin, imFrame.xmax, imFrame.ymin, imFrame.ymax, M.ref_ra, M.ref_dec);
+
+	    // mod jrichard shapemodel_binning
+	    sizex=imFrame.nx*O.bin;
+            sizey=imFrame.ny*O.bin;
+	    // mod bclement shapemodel_contour
+	    //o_pixel(ero, imFrame.nx, imFrame.ny, imFrame.pixelx, imFrame.xmin, imFrame.xmax, imFrame.ymin, imFrame.ymax, source, map_axx, map_ayy);
+	    o_pixel_contour(ero, sizex,sizey, imFrame.pixelx, imFrame.xmin, imFrame.xmax, imFrame.ymin, imFrame.ymax, source, imo_pl_ext, imo_pl_dpl_ext, imo_npl_ext);  // mod bclement shapemodel_contour_ext
+
+            if (O.setbin)
+                d_binning(ero,&sizex,&sizey, O.bin);
 
             if (O.setseeing)
                 d_seeing_omp(ero, imFrame.nx, imFrame.ny, imFrame.pixelx);
@@ -194,28 +210,42 @@ int o_chi_lhood0(double *chi_ext, double *lhood0_ext, double *np_b0)
             if (O.bruit)
                 d_bruiter_omp(ero, imFrame.nx, imFrame.ny);
 
+	    //wrf_fits_abs("tmp_binned_chi2.fits", ero, imFrame.nx, imFrame.ny, imFrame.xmin, imFrame.xmax, imFrame.ymin, imFrame.ymax, M.ref_ra, M.ref_dec);
+
             M.verbose = verbose_save;
-	
+
             if (wo != NULL)
             {
-                for (i = 0; i < imFrame.ny; i++ )
-                    for (j = 0; j < imFrame.nx; j++ )
-                    {
-                        ero[i][j] -= imo[i][j];
-                        chi_im += ero[i][j] * ero[i][j] * wo[i][j];
-                    }
+		// mod bclement shapemodel_contour
+		// compute the chi2 only inside the contours
+                //for (i = 0; i < imFrame.ny; i++ )
+		//for (j = 0; j < imFrame.nx; j++ )
+		for(n = 0; n < imo_npl; n++)
+		{
+		    i = imo_pl[n].ii;
+		    j = imo_pl[n].jj;
+		    ero[i][j] -= imo[i][j];
+		    chi_im += ero[i][j] * ero[i][j] * wo[i][j];
+		}
+	        //wrf_fits_abs("tmp_residuals.fits", ero, imFrame.nx, imFrame.ny, imFrame.xmin, imFrame.xmax, imFrame.ymin, imFrame.ymax, M.ref_ra, M.ref_dec);
+		
                 lhood0 = lhood_wo;  // read from lhood_wo variable pre-computed in o_global()
             }
             else
             {
-                for (i = 0; i < imFrame.ny; i++ )
-                    for (j = 0; j < imFrame.nx; j++ )
-                    {
-                        ero[i][j] -= imo[i][j];
-                        chi_im += ero[i][j] * ero[i][j] / fabs(imo[i][j] + 1.);
-                        lhood0 += log( 2.*M_PI*fabs(imo[i][j] + 1.) );
-                    }
-            }
+		// mod bclement shapemodel_contour
+		// compute the chi2 only inside the contours
+                //for (i = 0; i < imFrame.ny; i++ )
+                //    for (j = 0; j < imFrame.nx; j++ )
+		for(n = 0; n < imo_npl; n++)
+		{
+		    i = imo_pl[n].ii;
+		    j = imo_pl[n].jj;
+		    ero[i][j] -= imo[i][j];
+		    chi_im += ero[i][j] * ero[i][j] / fabs(imo[i][j] + 1.);
+		    lhood0 += log( 2.*PI*fabs(imo[i][j] + 1.) );
+		}
+	    }
 
             // restore relative source positions
             if( I.n_mult > 0 )
@@ -300,7 +330,7 @@ int o_chi_lhood0(double *chi_ext, double *lhood0_ext, double *np_b0)
                     {
                         ero[i][j] -= imo[i][j];
                         chi_im += ero[i][j] * ero[i][j] / fabs(imo[i][j] + 1.);
-                        lhood0 += log( 2.*M_PI*fabs(imo[i][j] + 1.) );
+                        lhood0 += log( 2.*PI*fabs(imo[i][j] + 1.) );
                     }
             }
             
@@ -345,7 +375,7 @@ int o_chi_lhood0(double *chi_ext, double *lhood0_ext, double *np_b0)
             ampli = e_unmag(&shm[i].C, I.dl0ssh, I.dossh, I.zsh);
             qp = fabs(ampli.b / ampli.a);
             tp = fabs(1. - qp * qp) / 2. / qp;
-            ampli.theta += M_PI / 2.;
+            ampli.theta += PI / 2.;
             x = (shm[i].mx - tp * cos(2.*ampli.theta)) / shm[i].err;
             chish += x * x;
             x = (shm[i].my - tp * sin(2.*ampli.theta)) / shm[i].err;
@@ -463,7 +493,7 @@ if (Dy.dynnumber == 1 || Dy.dynnumber == 2 || Dy.dynnumber == 3 || Dy.dynnumber
 		
 	    chi_vel = (Dy.dynvel - vel_model) * (Dy.dynvel - vel_model) / Dy.dynevel / Dy.dynevel;
 		
-	    lhood0 +=  log( 2.*M_PI*(Dy.dynevel*Dy.dynevel) );    // NORMALIZATION
+	    lhood0 +=  log( 2.*PI*(Dy.dynevel*Dy.dynevel) );    // NORMALIZATION
 		
 	}	
 	
@@ -476,7 +506,7 @@ if (Dy.dynnumber == 1 || Dy.dynnumber == 2 || Dy.dynnumber == 3 || Dy.dynnumber
 		
 		chi_mass = (Dy.indmass - mass_model)*(Dy.indmass - mass_model) / Dy.indemass / Dy.indemass;
 		
-		lhood0 +=  log( 2.*M_PI*(Dy.indemass*Dy.indemass) );    // NORMALIZATION
+		lhood0 +=  log( 2.*PI*(Dy.indemass*Dy.indemass) );    // NORMALIZATION
 		
 	}
 	
@@ -488,7 +518,7 @@ if (Dy.dynnumber == 1 || Dy.dynnumber == 2 || Dy.dynnumber == 3 || Dy.dynnumber
 		
 		chi_mass = (Dy.indmass - mass_model)*(Dy.indmass - mass_model) / Dy.indemass / Dy.indemass;
 		
-		lhood0 +=  log( 2.*M_PI*(Dy.indemass*Dy.indemass) );    // NORMALIZATION
+		lhood0 +=  log( 2.*PI*(Dy.indemass*Dy.indemass) );    // NORMALIZATION
 		
 	}
 	
@@ -742,8 +772,8 @@ static void chi2_wl(double *chi2, double *lh0)
             chi2_add += chi2i;
             //if ( I.statmode == 1 || I.dsigell != -1 )
             //{
-            lh0_add += log( 2 * M_PI * (I.sig2ell + arclet[i].var1) ) +
-                       log( 2 * M_PI * (I.sig2ell + arclet[i].var2) );
+            lh0_add += log( 2 * PI * (I.sig2ell + arclet[i].var1) ) +
+                       log( 2 * PI * (I.sig2ell + arclet[i].var2) );
 
             //}
 
@@ -775,7 +805,7 @@ static void chi2_wl(double *chi2, double *lh0)
 //       wht_ell = 1. / I.sig2ell;
 
         // Compute the normalisation factor
-//        *lh0 += narclet * log( 2 * M_PI * I.sig2ell );
+//        *lh0 += narclet * log( 2 * PI * I.sig2ell );
         double lh0_add  = 0;
         double chi2_add = 0;
         int num_threads = 1;
@@ -843,8 +873,8 @@ static void chi2_wl(double *chi2, double *lh0)
             chi2i += es2 * es2 / (I.sig2ell + arclet[i].var2 );
 
             chi2_add += chi2i;
-            lh0_add += log( 2 * M_PI * (I.sig2ell + arclet[i].var1) ) +
-                       log( 2 * M_PI * (I.sig2ell + arclet[i].var2) );
+            lh0_add += log( 2 * PI * (I.sig2ell + arclet[i].var1) ) +
+                       log( 2 * PI * (I.sig2ell + arclet[i].var2) );
 
 #ifdef CHIRES
 #pragma omp critical
@@ -1038,7 +1068,7 @@ static int chi2_src( double *chi2, double *lh0, double *np_b0 )
 //          dx=sqrt(sig2pos[i][j])-sigposMean;
 //          chip+=dx*dx/I2x;
 //      }
-//  *lh0 += log( 2.*M_PI*I2x );
+//  *lh0 += log( 2.*PI*I2x );
 //
     // It's not very necessary for I.forme==5 but it can be worth that
     // in the rest of the code all the images have their amplification
@@ -1166,7 +1196,7 @@ static int chi2_src( double *chi2, double *lh0, double *np_b0 )
            the observed image position.
          */
         int det_stop = 0;
-        nimages = unlens_bc(Ps, Bs, multi[i], multib, I.mult[i], i, &det_stop);
+        //nimages = unlens_bc(Ps, Bs, multi[i], multib, I.mult[i], i, &det_stop);
 
 
 #endif
@@ -1244,8 +1274,8 @@ static int chi2_src( double *chi2, double *lh0, double *np_b0 )
 //                    chipij = dx * dx * w2x + dy * dy * w2y + 2. * dx * dy * wxy;
                     chipij = dx_i * dx_i * wi2x + dy_i * dy_i * wi2y + 2. * dx_i * dy_i * wixy;
                     // update normalisation factor for image ij
-//                    lhood0 += log( 4.*M_PI * M_PI / (w2x * w2y - wxy * wxy));
-                    lhood0 += log( 4.*M_PI * M_PI / (wi2x * wi2y - wixy * wixy));
+//                    lhood0 += log( 4.*PI * PI / (w2x * w2y - wxy * wxy));
+                    lhood0 += log( 4.*PI * PI / (wi2x * wi2y - wixy * wixy));
 
                     // Detect infinite amplification issues
                     //if ( chipij < 0. ) return(1); // error
@@ -1285,7 +1315,7 @@ static int chi2_src( double *chi2, double *lh0, double *np_b0 )
                         I2y = Iy * Iy;
 
                         chipij = dx * dx / I2x + dy * dy / I2y;
-                        lhood0 += log( 4.*M_PI * M_PI * I2x * I2y);
+                        lhood0 += log( 4.*PI * PI * I2x * I2y);
                 }
                 else
                 {
@@ -1325,7 +1355,7 @@ static int chi2_src( double *chi2, double *lh0, double *np_b0 )
 
                     chipij = dx * dx * w2x + dy * dy * w2y + 2. * dx * dy * wxy;
                     // update normalisation factor for image ij
-                    lhood0 += log( 4.*M_PI * M_PI / (w2x * w2y - wxy * wxy));
+                    lhood0 += log( 4.*PI * PI / (w2x * w2y - wxy * wxy));
                 }
 
                 // sum total chip
@@ -1369,7 +1399,7 @@ static int chi2_src( double *chi2, double *lh0, double *np_b0 )
                     double da, sig2flux;
                     o_chi_flux(&multi[i][j], fluxS, &da, &sig2flux);
                     chia += da * da / sig2flux;
-                    lhood0 += log( 2.*M_PI * sig2flux );
+                    lhood0 += log( 2.*PI * sig2flux );
 #ifdef CHIRES
                     chiaij = da * da / sig2flux;
                     chiai += chiaij;
@@ -1379,9 +1409,9 @@ static int chi2_src( double *chi2, double *lh0, double *np_b0 )
 #ifdef CHIRES
                 // compute the error in image plane from the multib[]
                 // computed earlier
-                dx = multib[j].x - multi[i][j].C.x;
-                dy = multib[j].y - multi[i][j].C.y;
-                rmsij = dx * dx + dy * dy;
+                //dx = multib[j].x - multi[i][j].C.x;
+                //dy = multib[j].y - multi[i][j].C.y;
+                rmsij = 0.0;//dx * dx + dy * dy;
                 int warnij = 0;
                 if ( rmsij > 0 )
                     rmsi += rmsij;
@@ -1433,7 +1463,7 @@ static int chi2_src( double *chi2, double *lh0, double *np_b0 )
 
     // update the total statistics
     if ( I.forme == 12 )
-        *lh0 += ndxdy * log(2. * M_PI) + log(I.detCov);
+        *lh0 += ndxdy * log(2. * PI) + log(I.detCov);
     else
         *lh0 += lhood0;
 
@@ -1448,6 +1478,7 @@ static int chi2_img( double *chi2, double *lh0 )
     extern struct g_mode    M;
 #endif
     const extern struct g_image   I;
+    extern struct g_source   S;
     extern double chip, chia, chix, chiy;
     extern struct galaxie   multi[NFMAX][NIMAX];
     extern int nwarn; /*counts each time a barycenter is not found in a
@@ -1509,10 +1540,11 @@ static int chi2_img( double *chi2, double *lh0 )
     for (i = 0; i < I.n_mult; i++)
     {
         struct point Ps[NFMAX + 1];
-        /* optim_z can be different from 0 with the o_runz*() functions*/
-        if (optim_z == 0)
+        /* optim_z can be different from 0 withchi2_img the o_runz*() functions*/
+        if (optim_z == 0){
             /* compute the sources positions in Ps*/
-            o_dpl(I.mult[i], multi[i], Ps, NULL);
+            o_dpl(I.mult[i], multi[i], Ps, NULL); 
+        }
         else
         {
             for (j = 0; j < I.mult[i]; j++)
@@ -1527,6 +1559,11 @@ static int chi2_img( double *chi2, double *lh0 )
         /*vBs[i] contains the barycenter of the I.mult[i] sources positions Ps*/
 	if (I.forme == -2 )
             vBs[i] = weight_baryc_amp(Ps, multi[i], I.mult[i], i); /* amplification-weighted barycenter (BCLEMENT)*/
+        /* test source fixé benjamin beauchesne */
+        else if (I.forme == -3 ){
+            vBs[i]=S.multi_s[i];        
+        }
+        
         else
             vBs[i] = bcentlist(Ps, I.mult[i]);  /* barycentre */
 
@@ -1644,10 +1681,9 @@ static int chi2_img( double *chi2, double *lh0 )
 
                 struct galaxie *pima = &multi[i][j];
 
-                if ( I.forme == -1 || I.forme == -2)
+                if ( I.forme == -1 || I.forme == -2 || I.forme == -3)
                 {
                     I2x = I2y = I.sig2pos[i][j];
-		    chip_penalty = ( chip_penalty_dist * chip_penalty_dist ) / I2x; // penalty function of position error
                 }
                 else if ( I.forme == -10 )
                 {
@@ -1660,9 +1696,10 @@ static int chi2_img( double *chi2, double *lh0 )
                     I2x = I2x * I2x;
                     I2y = I2y * I2y;
                 }
+		        chip_penalty = ( chip_penalty_dist * chip_penalty_dist ) / I2x; // penalty function of position error
 
                 // update normalisation factor for image ij
-                dLhood0 += log( 4.*M_PI * M_PI * (I2x * I2y) );
+                dLhood0 += log( 4.*PI * PI * (I2x * I2y) );
 
                 dx = vmultib[task_idx].x - multi[i][j].C.x;
                 dy = vmultib[task_idx].y - multi[i][j].C.y;
@@ -1819,7 +1856,7 @@ static void srcfitLinFit(double *chi2, double *lh0)
         fprintf(OUT, "%2d  %4s    %.3lf    %.2lf\n", i, srcfit[i].n, wt[i], tmp*tmp ); // (1. + a * a));
 #endif
         *chi2 += tmp * tmp ;   /// ( 1.  + a * a);
-        *lh0 += log( 2.*M_PI / wt[i] / wt[i] );
+        *lh0 += log( 2.*PI / wt[i] / wt[i] );
     }
 
 #ifdef CHIRES
