@@ -14,44 +14,10 @@
 #include "omp.h"
 #endif
 
+#include "o_global.h"
+
 static void readWeightMat(double *** weight, double *detCov);
 
-struct shear   shm[NAMAX];
-struct galaxie multi[NFMAX][NIMAX];
-struct z_lim   zlim[NFMAX];
-struct z_lim   zalim;
-struct galaxie *srcfit;  // points for source plane fitting
-struct z_lim   zlim_s[NFMAX];
-struct pot     lmin_s[NLMAX], lmax_s[NLMAX], prec_s[NLMAX];
-struct sigposStr sigposAs_s, sigposAs;
-struct matrix  amplifi_mat[NFMAX][NIMAX], amplifi_matinv[NFMAX][NIMAX];
-//struct pixlist plo[100000]; /*list of points that compose the polygon around the image to inverse DEPRECATED*/
-
-double z_dlsds;
-double chi_im, chip, chix, chiy, chia, chil, chis, chi_vel,chi_mass;    //I added chi_vel and chi_mass TV
-//double amplifi[NFMAX][NIMAX];
-double **imo, **wo, **soo, **ero, lhood_wo;
-double ***cubeo,***ercubeo;
-
-double drima;   /* distance ratio between the image to inverse and the main lens*/
-double **sm_p;      /* optimization spline map : initial save */
-double *sflux;
-double **fluxmap;
-int **nimage;
-int    nshmap;
-int    optim_z;
-int    block_s[NLMAX][NPAMAX];
-int  **imuo;
-/*number of points of the polygon around the image to inverse*/
-
-double distmin[NFMAX]; /* List of minimal euclidean distances between
-                          the images of a same familly i*/
-
-int    nwarn;   // warning emitted during image plane chi2
-
-double *np_b0;  // non parametric accelerator
-//int    nplo;    // number of points for criticinv.c DEPRECATED
-
 /****************************************************************/
 /*      nom:        global              */
 /*      auteur:     Jean-Paul Kneib         */
@@ -109,7 +75,7 @@ void  o_global()
 //      exit(-1);
 //      return;
 //    }
-
+    
     // Start the optimisation engines
     if ( M.inverse == 3 || M.inverse == 4 )
         evidence = o_run_bayes();
@@ -117,6 +83,10 @@ void  o_global()
      {
 //        evidence = o_run_nest();
      }
+//   else if (M.inverse == 6)
+//     {
+//	run_servermode();
+//     }
     else
     {
         printf("ERROR : inverse modes 1 & 2 are not supported anymore\n");
@@ -184,16 +154,16 @@ void  o_global()
 
     /*-------------------------------------------------------
      * Reset the lens parameters and print the optimisation results */
-
+    printf("%f %f \n",lens[0].b0,lens[1].b0);
     /*Fill the chires.dat file*/
     o_chires("chires.dat");
-
+    printf("%f %f \n",lens[0].b0,lens[1].b0);
     /*Reset the lens parameters*/
     set_res_par();
-
+    printf("%f %f \n",lens[0].b0,lens[1].b0);
     /*Print the results of optimisation*/
     o_print_res(o_chi(), evidence);
-
+    printf("%f %f \n",lens[0].b0,lens[1].b0);
 //  NPRINTF(stderr,"chi: %.1lf (%.1lf,%.1lf)\n",chi0,chip,chil);
 
     /*-------------------------------------------------------*/
@@ -271,7 +241,11 @@ void o_global_free()
         //  free the square maps
         if (M.iclean == 2 )
         {
-            free_square_double(ero, imFrame.ny);
+	    const extern struct g_observ  O; 
+            free_square_double(ero, imFrame.ny*O.bin);
+	    free(imo_pl); // mod bclement shapemodel_contour
+	    free(imo_pl_ext); // mod bclement shapemodel_contour_ext
+	    free(imo_pl_dpl_ext); // mod bclement shapemodel_contour_ext
         }
         else if (M.iclean == 3 )
         {
@@ -448,7 +422,7 @@ void readConstraints()
             for( i = 0; i < wFrame.nx; i++ )
                 for( j = 0; j < wFrame.ny; j++ )
                     if( wo[j][i] > 0 )
-                        lhood_wo -= log( 2.*M_PI*wo[j][i] );  
+                        lhood_wo -= log( 2.*PI*wo[j][i] );  
         }
 
 
@@ -457,9 +431,69 @@ void readConstraints()
 
             if(M.iclean==2|| M.iclean ==4)
             {
-              // allocate temporary array to compute chi2 in o_chi()
-              ero = (double **) alloc_square_double(imFrame.ny, imFrame.nx);
+		const extern struct g_observ  O; // mod bclement shapemodel_contour_ext
+                // allocate temporary array to compute chi2 in o_chi()
+                ero = (double **) alloc_square_double(imFrame.ny*O.bin, imFrame.nx*O.bin);
             }
+
+	    // mod bclement shapemodel_contour
+	    // Initialize and build the pixlist defining the contours
+	    if(M.iclean == 2)
+	    {
+		const extern struct g_observ  O; // mod bclement shapemodel_contour_ext
+
+		/* Allocate the maximum number of points to send to source plane */
+		imo_pl = (struct pixlist*)malloc((unsigned int)
+                                 imFrame.nx * imFrame.ny * sizeof(struct pixlist));
+
+		/*  keep only the relevant points */
+		if (imFrame.ncont == 0)
+		{
+		    NPRINTF(stderr, "ERROR: the number of contours is %d (not defined). Set the keyword ncont in the parameter file.\n",imFrame.ncont);
+		    exit(-1);
+		}
+		else
+		{
+		    keep_cl(imo, imo_pl, &imo_npl);
+		    // mod bclement shapemodel_contour_ext
+		    if (O.setseeing)
+		    {
+			imo_pl_ext = (struct pixlist*)malloc((unsigned int)
+                                 O.bin*O.bin*imFrame.nx * imFrame.ny * sizeof(struct pixlist));
+			keep_cl_extended(imo_pl, imo_npl, imo_pl_ext, &imo_npl_ext);
+		    }
+		    else
+		    {
+                        imo_pl_ext = (struct pixlist*)malloc((unsigned int)
+                                 imFrame.nx * imFrame.ny * sizeof(struct pixlist));
+                        keep_cl(imo, imo_pl_ext, &imo_npl_ext);
+		    }
+		}
+
+		// TODO : remove duplicate pixels in list when overlapping contours occur
+		// This would cause a bug o_pixel_contour()
+
+		// TODO possibly reallocate imo_pl given imo_npl 
+		imo_pl = (struct pixlist*)realloc(imo_pl, (unsigned int)
+                                 imo_npl * sizeof(struct pixlist));
+
+		// mod bclement shapemodel_contour_ext
+		imo_pl_ext = (struct pixlist*)realloc(imo_pl_ext, (unsigned int)
+                                 imo_npl_ext * sizeof(struct pixlist));
+
+		// redo the pre-computation of the loglikelihood normalization factor
+		lhood_wo = 0.;
+		int n;
+		for( n = 0; n < imo_npl; n++ )
+		{
+		    j = imo_pl[n].ii;
+		    i = imo_pl[n].jj;
+                    if( wo[j][i] > 0 )
+                        lhood_wo -= log( 2.*PI*wo[j][i] );  
+		}
+
+	    }
+
             if(M.iclean==3)
             {
               // allocate temporary array to compute chi2 in o_chi()
@@ -482,22 +516,41 @@ void readConstraints()
 
             if( G.no_lens == 0 && pot_nopt == 0 )
             {
-                map_axx = (double **) alloc_square_double(imFrame.ny, imFrame.nx);
-                map_ayy = (double **) alloc_square_double(imFrame.ny, imFrame.nx);
+                //mod bclement shapemodel_contour_ext
                 struct point pi, ps;
                 double zs_ref = 100.;
                 double dlsds_ref = dratio(lens[0].z, zs_ref);
-                for (i = 0; i < imFrame.ny; i++)
-                {
-                    pi.y = imFrame.ymin + i * imFrame.pixelx;
-                    for (j = 0; j < imFrame.nx; j++)
-                    {
-                        pi.x = imFrame.xmin + j * imFrame.pixelx;
-                        e_dpl(&pi, dlsds_ref, zs_ref, &ps);
-                        map_axx[i][j] = pi.x - ps.x;
-                        map_ayy[i][j] = pi.y - ps.y;
-                    }
-                }
+		if(M.iclean == 2) // compute normalized dpl only inside extended contour
+		{
+		    imo_pl_dpl_ext = (struct pixlist*)malloc((unsigned int)
+			      imo_npl_ext * sizeof(struct pixlist));
+		    int n;
+		    for(n = 0; n < imo_npl_ext; n++)
+		    {
+			pi.y = imo_pl_ext[n].i;
+			pi.x = imo_pl_ext[n].j;
+			e_dpl(&pi, dlsds_ref, zs_ref, &ps);
+			imo_pl_dpl_ext[n].i = (pi.x - ps.x)/dlsds_ref;
+			imo_pl_dpl_ext[n].j = (pi.y - ps.y)/dlsds_ref;
+		    }
+		}
+		else
+		{
+		    map_axx = (double **) alloc_square_double(imFrame.ny, imFrame.nx);
+		    map_ayy = (double **) alloc_square_double(imFrame.ny, imFrame.nx);
+		    
+		    for (i = 0; i < imFrame.ny; i++)
+		    {
+			pi.y = imFrame.ymin + i * imFrame.pixelx;
+			for (j = 0; j < imFrame.nx; j++)
+			{
+			    pi.x = imFrame.xmin + j * imFrame.pixelx;
+			    e_dpl(&pi, dlsds_ref, zs_ref, &ps);
+			    map_axx[i][j] = pi.x - ps.x;
+			    map_ayy[i][j] = pi.y - ps.y;
+			}
+		    }
+		}
             }
 
             // read and prepare sources for o_chi()
@@ -685,7 +738,26 @@ void readConstraints()
         }
         free(mult);
     }
+    
+    //benjamin Beauchesne teste true source
+     
+    if (S.multmode != 0)
+    {
+        long int ntmult_s = 0; 
+        struct galaxie *mult_s = (struct galaxie *) malloc((unsigned long int) NFMAX * NIMAX * sizeof(struct galaxie));
+        if ( mult_s == NULL )
+        {
+            fprintf( stderr, "ERROR: mult[NFMAX*NIMAX] memory allocation failed.\n");
+            exit(-1);
+        }
 
+        f_shape(&ntmult_s, mult_s, S.multfile, S.multmode);
+        sort(ntmult_s, mult_s, comparer_z);
+        for (i=0;i<ntmult_s;i++){
+            S.multi_s[i]=mult_s[i].C;
+        }
+        //free(mult_s);
+    }
     // Prepare Accelerated potentials
     if (( G.nmsgrid != G.nlens ) || (M.iclean == 4))
     {
@@ -749,6 +821,9 @@ int getNConstraints()
     extern struct g_image I;
     extern struct g_dyn   Dy;   //TV Oct 2011
     extern long int narclet;
+    extern struct pixlist *imo_pl; // mod bclement shapemodel_contour
+    extern double **wo; // mod bclement shapemodel_contour
+    extern int imo_npl; // mod bclement shapemodel_contour
     int constraints, i;
     int opi;        // number of observable per images
 
@@ -795,8 +870,20 @@ int getNConstraints()
     // FITS image reconstruction
     if ( M.iclean == 2 )
     {
-        extern struct g_pixel imFrame;
-        constraints += imFrame.nx * imFrame.ny;
+        //extern struct g_pixel imFrame;
+        //constraints += imFrame.nx * imFrame.ny;
+	// mod bclement shapemodel_contour
+	// Number of constraints is now the number 
+	// of non-zero weight pixels inside the contours
+	int n,j;
+	for( n = 0; n < imo_npl; n++ )
+	{
+	    j = imo_pl[n].ii;
+	    i = imo_pl[n].jj;
+	    if( wo[j][i] > 0 )
+		constraints++;
+	}
+
     }
 
     //CHECK IF M.iclean = 4
@@ -833,17 +920,25 @@ int getNParameters()
         parameters = ps.nx*ps.ny;
     }
     
-    for ( i = 0; i < G.no_lens; i ++ )
-        for ( ipx = CX; ipx <= PMASS; ipx ++ )
-            if ( block[i][ipx] != 0 )
+    for ( i = 0; i < G.no_lens; i ++ ){
+        for ( ipx = CX; ipx <= PMASS; ipx ++ ){
+            if ( block[i][ipx] != 0 ){
                 parameters++;
-
+            }
+        }
+        //Benjamin Beauchesne - Bspline modification
+        for ( ipx = CX_PERT; ipx < NPAMAX; ipx ++ ){
+            if ( block[i][ipx] != 0 ){
+                parameters++;
+            }
+        }
+    };
     // multiscale grid clumps
     parameters += G.nlens - G.nmsgrid;
 
     // source parameters
     for ( i = 0; i < S.ns; i++ )
-        for (ipx = SCX; ipx <= SFLUX; ipx++ )
+        for (ipx = SCX; ipx <= SFLUX2; ipx++ )
             if ( sblock[i][ipx] != 0 )
                 parameters++;
 
