@@ -22,6 +22,7 @@
 
 static void writePotentiel(FILE *best, long int i, int flag);
 static void writeLimit(FILE *best, long int i);
+static void writeBspline_coeff(struct Bspline2D Bspline,int i);
 
 void  o_print_res(double chi0, double evidence)
 {
@@ -61,7 +62,7 @@ void  o_print_res(double chi0, double evidence)
     extern double z_dlsds;
     extern double chip, chix, chiy, chis, chil, chi_vel,chi_mass;    //I added chi_vel and chi_mass TV
     extern double **map_p, **map_axx, **map_ayy;
-    extern struct g_pixel ps, imFrame;
+    extern struct g_pixel ps, imFrame, wFrame;
     int     i, j;
     FILE    *best, *besto;
     char    limages[ZMBOUND][IDSIZE];
@@ -155,8 +156,8 @@ void  o_print_res(double chi0, double evidence)
     fprintf(best, "#Chi2pos: %.6lf\n", chip);
 	fprintf(best, "#Chi2_vel: %.6lf\n",chi_vel);      //THIS IS MINE
 	fprintf(best, "#Chi2_mass: %.6lf\n",chi_mass);      //THIS IS MINE
-    fprintf(best, "#Chi2formex: %.6lf\n", chix);
-    fprintf(best, "#Chi2formey: %.6lf\n", chiy);
+    fprintf(best, "#Chi2formx: %.6lf\n", chix);
+    fprintf(best, "#Chi2formy: %.6lf\n", chiy);
     fprintf(best, "#Chi2l: %.6lf\n", chil);
     if ( M.inverse == 3 )
         fprintf( best, "#log(Evidence): %.6lf\n", evidence);
@@ -193,17 +194,17 @@ void  o_print_res(double chi0, double evidence)
     if (M.ishearf)
         fprintf(best, "\tshearfield  %d %.6lf %s %d\n", M.ishearf, M.zshearf, M.shearffile, M.nshearf);
     if (M.grille)
-        fprintf(best, "\tgrille   %d %d %lf\n", M.grille, M.ngrille, M.zgrille);
+        fprintf(best, "\tgrid     %d %d %lf\n", M.grille, M.ngrille, M.zgrille);
     if (M.pixel)
         fprintf(best, "\tpixel     %d %d %s\n", M.pixel, M.npixel, M.pixelfile);
 
     fprintf(best, "\tend\n");
 
     // GRILLE
-    fprintf(best, "grille\n");
-    fprintf(best, "\tnombre      %d\n", G.ngrid);
-    fprintf(best, "\tpolaire     %d\n", 0);
-    fprintf(best, "\tnlentille   %ld\n", G.nlens);
+    fprintf(best, "grid\n");
+    fprintf(best, "\tnumber      %d\n", G.ngrid);
+    fprintf(best, "\tpolar     %d\n", 0);
+    fprintf(best, "\tnlens   %ld\n", G.nlens);
     if ( strcmp(CL.algorithm, "MARCHINGSQUARES") )
         fprintf(best, "\tnlens_crit   %ld\n", G.nlens_crit);
     if ( sigposAs.bk != 0 )
@@ -249,6 +250,8 @@ void  o_print_res(double chi0, double evidence)
         fprintf(best, "\tcleanset  %d  %f\n", M.iclean, M.zclean);
         if(strcmp(imFrame.pixfile, ""))
             fprintf(best, "\timframe %d  %s\n", imFrame.format, imFrame.pixfile);
+        if(strcmp(wFrame.pixfile, ""))
+            fprintf(best, "\twframe %d  %s\n", wFrame.format, wFrame.pixfile);
         if(strcmp(ps.pixfile, "")) 
             fprintf(best, "\tsframe  %s\n", ps.pixfile);
         if(strcmp(M.centerfile, ""))
@@ -277,6 +280,33 @@ void  o_print_res(double chi0, double evidence)
         fprintf(best, "\tend\n");
     }
 
+    if ( M.iclean == 2 )
+        for( i = 0; i < S.ns; i++ )
+        {
+            // SHAPE MODEL
+            fprintf(best, "shapemodel\n");
+            fprintf(best, "\tid   %s\n",  source[i].n);
+            fprintf(best, "\ttype         %d\n",     source[i].type);
+            fprintf(best, "\ts_center_x   %.6lf\n",  source[i].C.x);
+            fprintf(best, "\ts_center_y   %.6lf\n",  source[i].C.y);
+            fprintf(best, "\ts_sigx       %.6lf\n",  source[i].E.a);
+            fprintf(best, "\ts_sigy       %.6lf\n",  source[i].E.b);
+            fprintf(best, "\ts_eps        %.6lf\n",  source[i].eps);
+            fprintf(best, "\ts_angle      %.6lf\n",  source[i].E.theta * RTD);
+            fprintf(best, "\ts_mag          %.6lf\n",  source[i].mag);
+            fprintf(best, "\ts_index        %.6lf\n",  source[i].var1);
+	    if(source[i].type==6) // double Sersic
+            {
+            	fprintf(best, "\ts_sigx2      %.6lf\n",  source[i].E2.a);
+            	fprintf(best, "\ts_sigy2      %.6lf\n",  source[i].E2.b);
+            	fprintf(best, "\ts_eps2       %.6lf\n",  source[i].eps2);
+            	fprintf(best, "\ts_angle2     %.6lf\n",  source[i].E2.theta * RTD);
+            	fprintf(best, "\ts_mag2         %.6lf\n",  source[i].mag2);
+            	fprintf(best, "\ts_index2       %.6lf\n",  source[i].var2);
+	    }
+            fprintf(best, "\tend\n");
+        }
+
     // Write all the potentials with arcsec and kpc values
     for (i = 0; i < G.nlens; i++)
         writePotentiel(best, i, 3);
@@ -317,7 +347,7 @@ void  o_print_res(double chi0, double evidence)
     fprintf(best, "grande\n");
     fprintf(best, "\tiso         %d %d %.6lf %.6lf %.6lf\n", L.iso, L.nmaxiso, L.scale, L.zonex, L.zoney);
     fprintf(best, "\tname        best\n");
-    fprintf(best, "\tprofil      %d %d\n", L.profil, L.pt);
+    fprintf(best, "\tprofile      %d %d\n", L.profil, L.pt);
     fprintf(best, "\tcontour     %d %d\n", L.ncourbe, L.pt);
     fprintf(best, "\tlarge_dist  %.6lf\n", L.dlarge);
     fprintf(best, "\tend\n");
@@ -334,7 +364,7 @@ void  o_print_res(double chi0, double evidence)
             fprintf(best, "\tpsf      %d %s\n", O.setseeing, O.psffile);
 
         fprintf(best, "\tbinning      %d %d\n", O.setbin, O.bin);
-        fprintf(best, "\tbruit        %d\n", O.bruit);
+        fprintf(best, "\tnoise        %d\n", O.bruit);
         fprintf(best, "\tSKY          %.6lf\n", O.SKY);
         if( O.gain > 0 )
             fprintf(best, "\tdispersion   %.6lf\n", sqrt(O.SKY / O.gain));
@@ -360,7 +390,7 @@ void  o_print_res(double chi0, double evidence)
     }
 
     // COSMOLOGY
-    fprintf(best, "cosmologie\n");
+    fprintf(best, "cosmology\n");
     fprintf(best,"\tmodel       %d\n",C.model);
     fprintf(best, "\tH0        %.6lf\n", C.H0);
     fprintf(best, "\tomegaM    %.6lf\n", C.omegaM);
@@ -371,7 +401,7 @@ void  o_print_res(double chi0, double evidence)
     fprintf(best, "\tend\n");
 
     // CHAMP
-    fprintf(best, "champ\n");
+    fprintf(best, "field\n");
     fprintf(best, "\txmin     %.6lf\n", F.xmin);
     fprintf(best, "\txmax     %.6lf\n", F.xmax);
     fprintf(best, "\tymin     %.6lf\n", F.ymin);
@@ -384,7 +414,7 @@ void  o_print_res(double chi0, double evidence)
     fprintf(best, "\tend\n");
 
 
-    fprintf(best, "fini\n");
+    fprintf(best, "finish\n");
     fclose(best);
 
 
@@ -413,8 +443,8 @@ void  o_print_res(double chi0, double evidence)
     fprintf(besto, "#Chi2pos: %.6lf\n", chip);
 	fprintf(besto, "#Chi2_vel: %.6lf\n",chi_vel);      //THIS IS MINE     TV
 	fprintf(besto, "#Chi2_mass: %.6lf\n",chi_mass);      //THIS IS MINE   TV
-    fprintf(besto, "#Chi2formex: %.6lf\n", chix);
-    fprintf(besto, "#Chi2formey: %.6lf\n", chiy);
+    fprintf(besto, "#Chi2formx: %.6lf\n", chix);
+    fprintf(besto, "#Chi2formy: %.6lf\n", chiy);
     fprintf(besto, "#Chi2l: %.6lf\n", chil);
     if ( M.inverse == 3 )
         fprintf( besto, "#log(Evidence): %.6lf\n", evidence);
@@ -445,7 +475,7 @@ void  o_print_res(double chi0, double evidence)
     if (M.ishearf)
         fprintf(besto, "\tshearfield  %d %.6lf %s %d\n", M.ishearf, M.zshearf, M.shearffile, M.nshearf);
     if (M.grille)
-        fprintf(besto, "\tgrille   %d %d %lf\n", M.grille, M.ngrille, M.zgrille);
+        fprintf(besto, "\tgrid   %d %d %lf\n", M.grille, M.ngrille, M.zgrille);
     if (M.pixel)
         fprintf(besto, "\tpixel     %d %d %s\n", M.pixel, M.npixel, M.pixelfile);
 
@@ -454,7 +484,7 @@ void  o_print_res(double chi0, double evidence)
     // IMAGE
     fprintf(besto, "image\n");
     fprintf(besto, "\tmultfile    %d %s\n", I.n_mult, I.multfile);
-    fprintf(besto, "\tforme       %d\n", I.forme);
+    fprintf(besto, "\tform       %d\n", I.forme);
     if (I.stat > 0)
         fprintf(besto, "\tarcletstat  %d %d %s\n", I.stat, I.statmode, I.arclet);
 
@@ -491,10 +521,10 @@ void  o_print_res(double chi0, double evidence)
     fprintf(besto, "\tend\n");
 
     // GRILLE
-    fprintf(besto, "grille\n");
-    fprintf(besto, "\tnombre      %d\n", G.ngrid);
-    fprintf(besto, "\tpolaire     %d\n", 0);
-    fprintf(besto, "\tnlentille   %ld\n", G.nlens);
+    fprintf(besto, "grid\n");
+    fprintf(besto, "\tnumber      %d\n", G.ngrid);
+    fprintf(besto, "\tpolar     %d\n", 0);
+    fprintf(besto, "\tnlens   %ld\n", G.nlens);
     if ( strcmp(CL.algorithm, "MARCHINGSQUARES") )
         fprintf(besto, "\tnlens_crit   %ld\n", G.nlens_crit);
 
@@ -515,7 +545,9 @@ void  o_print_res(double chi0, double evidence)
         fprintf(besto, "cleanlens\n");
         fprintf(besto, "\tcleanset  %d  %f\n", M.iclean, M.zclean);
         if(strcmp(imFrame.pixfile, ""))
-            fprintf(besto, "\timFrame %d  %s\n", imFrame.format, imFrame.pixfile);
+            fprintf(besto, "\timframe %d  %s\n", imFrame.format, imFrame.pixfile);
+        if(strcmp(wFrame.pixfile, ""))
+            fprintf(besto, "\twframe %d  %s\n", wFrame.format, wFrame.pixfile);
         if(strcmp(ps.pixfile, "")) 
             fprintf(besto, "\tsframe  %s\n", ps.pixfile);
         if(strcmp(M.centerfile, ""))
@@ -550,14 +582,24 @@ void  o_print_res(double chi0, double evidence)
             // SHAPE MODEL
             fprintf(besto, "shapemodel\n");
             fprintf(besto, "\tid   %s\n",  source[i].n);
+            fprintf(besto, "\ttype         %d\n",  source[i].type);
             fprintf(besto, "\ts_center_x   %.6lf\n",  source[i].C.x);
             fprintf(besto, "\ts_center_y   %.6lf\n",  source[i].C.y);
-            fprintf(besto, "\ts_angle      %.6lf\n",  source[i].E.theta * RTD);
             fprintf(besto, "\ts_sigx       %.6lf\n",  source[i].E.a);
             fprintf(besto, "\ts_sigy       %.6lf\n",  source[i].E.b);
-            fprintf(besto, "\tmag          %.6lf\n",  source[i].mag);
-            fprintf(besto, "\ttype         %d\n",  source[i].type);
-            fprintf(besto, "\tindex        %.6lf\n",  source[i].var1);
+            fprintf(besto, "\ts_eps        %.6lf\n",  source[i].eps);
+            fprintf(besto, "\ts_angle      %.6lf\n",  source[i].E.theta * RTD);
+            fprintf(besto, "\ts_mag          %.6lf\n",  source[i].mag);
+            fprintf(besto, "\ts_index        %.6lf\n",  source[i].var1);
+	    if(source[i].type==6) //double Sersic
+	    {
+            	fprintf(besto, "\ts_sigx2      %.6lf\n",  source[i].E2.a);
+            	fprintf(besto, "\ts_sigy2      %.6lf\n",  source[i].E2.b);
+            	fprintf(besto, "\ts_eps2       %.6lf\n",  source[i].eps2);
+            	fprintf(besto, "\ts_angle2     %.6lf\n",  source[i].E2.theta * RTD);
+            	fprintf(besto, "\ts_mag2         %.6lf\n",  source[i].mag2);
+            	fprintf(besto, "\ts_index2       %.6lf\n",  source[i].var2);
+	    }
             fprintf(besto, "\tend\n");
 
             // SHAPE LIMITS
@@ -566,24 +608,42 @@ void  o_print_res(double chi0, double evidence)
                 fprintf(besto, "\ts_center_x   %d %.6lf %.6lf\n", sblock[i][SCX], smin[i].C.x, smax[i].C.x);
             if( sblock[i][SCY] )
                 fprintf(besto, "\ts_center_y   %d %.6lf %.6lf\n", sblock[i][SCY], smin[i].C.y, smax[i].C.y);
-            if( sblock[i][STHETA] )
-                fprintf(besto, "\ts_angle      %d %.6lf %.6lf\n", sblock[i][STHETA], smin[i].E.theta * RTD, smax[i].E.theta * RTD);
             if( sblock[i][SA] )
                 fprintf(besto, "\ts_sigx       %d %.6lf %.6lf\n", sblock[i][SA], smin[i].E.a, smax[i].E.a);
-            if( sblock[i][SB] )
-             fprintf(besto, "\ts_sigy       %d %.6lf %.6lf\n", sblock[i][SB], smin[i].E.b, smax[i].E.b);
+            if( sblock[i][SEPS] )
+             fprintf(besto, "\ts_eps       %d %.6lf %.6lf\n", sblock[i][SEPS], smin[i].eps, smax[i].eps);
+            if( sblock[i][STHETA] )
+                fprintf(besto, "\ts_angle      %d %.6lf %.6lf\n", sblock[i][STHETA], smin[i].E.theta * RTD, smax[i].E.theta * RTD);
             if( sblock[i][SFLUX] )
-                fprintf(besto, "\tmag          %d %.6lf %.6lf\n", sblock[i][SFLUX], smin[i].mag, smax[i].mag);
+                fprintf(besto, "\ts_mag          %d %.6lf %.6lf\n", sblock[i][SFLUX], smin[i].mag, smax[i].mag);
             if( sblock[i][SINDEX] )
-                fprintf(besto, "\tindex        %d %.6lf %.6lf\n", sblock[i][SINDEX], smin[i].var1, smax[i].var1);
+                fprintf(besto, "\ts_index        %d %.6lf %.6lf\n", sblock[i][SINDEX], smin[i].var1, smax[i].var1);
+	    if(source[i].type==6) //double Sersic
+	    {
+            	if( sblock[i][SA2] )
+                	fprintf(besto, "\ts_sigx2       %d %.6lf %.6lf\n", sblock[i][SA2], smin[i].E2.a, smax[i].E2.a);
+            	if( sblock[i][SEPS2] )
+             		fprintf(besto, "\ts_eps2       %d %.6lf %.6lf\n", sblock[i][SEPS2], smin[i].eps2, smax[i].eps2);
+            	if( sblock[i][STHETA2] )
+                	fprintf(besto, "\ts_angle2      %d %.6lf %.6lf\n", sblock[i][STHETA2], smin[i].E2.theta * RTD, smax[i].E2.theta * RTD);
+            	if( sblock[i][SFLUX2] )
+                	fprintf(besto, "\ts_mag2          %d %.6lf %.6lf\n", sblock[i][SFLUX2], smin[i].mag2, smax[i].mag2);
+            	if( sblock[i][SINDEX2] )
+                	fprintf(besto, "\ts_index2        %d %.6lf %.6lf\n", sblock[i][SINDEX2], smin[i].var2, smax[i].var2);
+	    }
             fprintf(besto, "\tend\n");
         }
 
     // Write the optimized potentials and limits
+    //system("rm Coeff_best.dat");
     for ( i = 0 ; i < G.nplens[0] ; i++ )
     {
         writePotentiel(besto, i, 2);
         writeLimit(besto, i);
+        //Benjamin Beauchesne - Bspline modification
+        // to only write the file one time
+        //if ( lens[i].type == 814 || lens[i].type == 815) writeBspline_coeff(lens[i].Bspline);
+
     }
 
     // POTFILE
@@ -624,10 +684,11 @@ void  o_print_res(double chi0, double evidence)
         }
 
     // Write the grid potentials
+    //system("rm Coeff_best.dat");
     for( i = G.nmsgrid; i < G.nlens; i++ )
     {
         writePotentiel(besto, i, 2);
-    //    writeLimit(besto, i);
+            //    writeLimit(besto, i);
     }
 
     // CLINE
@@ -664,11 +725,12 @@ void  o_print_res(double chi0, double evidence)
     }
 
     // GRANDE
-    fprintf(besto, "grande\n");
+    fprintf(besto, "large\n");
     fprintf(besto, "\tiso         %d %d %.6lf %.6lf %.6lf\n", L.iso, L.nmaxiso, L.scale, L.zonex, L.zoney);
     fprintf(besto, "\tname        best\n");
-    fprintf(besto, "\tprofil      %d %d\n", L.profil, L.pt);
+    fprintf(besto, "\tprofile      %d %d\n", L.profil, L.pt);
     fprintf(besto, "\tcontour     %d %d\n", L.ncourbe, L.pt);
+    fprintf(besto, "\tvelocity    %d\n", L.vitesse);
     fprintf(besto, "\tlarge_dist  %.6lf\n", L.dlarge);
     fprintf(besto, "\tend\n");
 
@@ -684,7 +746,7 @@ void  o_print_res(double chi0, double evidence)
             fprintf(besto, "\tpsf      %d %s\n", O.setseeing, O.psffile);
 
         fprintf(besto, "\tbinning      %d %d\n", O.setbin, O.bin);
-        fprintf(besto, "\tbruit        %d\n", O.bruit);
+        fprintf(besto, "\tnoise        %d\n", O.bruit);
         fprintf(besto, "\tSKY          %.6lf\n", O.SKY);
         if( O.gain > 0 )
             fprintf(besto, "\tdispersion   %.6lf\n", sqrt(O.SKY / O.gain));
@@ -722,7 +784,7 @@ void  o_print_res(double chi0, double evidence)
     }
 
     // COSMOLOGY
-    fprintf(besto, "cosmologie\n");
+    fprintf(besto, "cosmology\n");
     fprintf(besto,"\tmodel       %d\n",C.model);
     fprintf(besto, "\tH0         %.6lf\n", C.H0);
     fprintf(besto, "\tomegaM     %.6lf\n", C.omegaM);
@@ -745,7 +807,7 @@ void  o_print_res(double chi0, double evidence)
     fprintf(besto, "\tend\n");
 
     // CHAMP
-    fprintf(besto, "champ\n");
+    fprintf(besto, "field\n");
     fprintf(besto, "\txmin     %.6lf\n", F.xmin);
     fprintf(besto, "\txmax     %.6lf\n", F.xmax);
     fprintf(besto, "\tymin     %.6lf\n", F.ymin);
@@ -757,7 +819,7 @@ void  o_print_res(double chi0, double evidence)
     }
     fprintf(besto, "\tend\n");
 
-    fprintf(besto, "fini\n");
+    fprintf(besto, "finish\n");
     fclose(besto);
 
     // Write the arclet.best file
@@ -773,7 +835,7 @@ void  o_print_res(double chi0, double evidence)
 }
 
 
-/* Write a potentiel section in best file 
+/* Write a potential section in best file 
  * flag = 1 : print values in arcsec
  * flag = 2 : print values in kpc
  * flag = 3 : print values in arcsec and kpc
@@ -782,8 +844,8 @@ static void writePotentiel(FILE *best, long int i, int flag)
 {
     extern struct pot lens[];
 
-    fprintf(best, "potentiel %s\n", lens[i].n);
-    fprintf(best, "\tprofil       %d\n", lens[i].type);
+    fprintf(best, "potential %s\n", lens[i].n);
+    fprintf(best, "\tprofile       %d\n", lens[i].type);
     if (lens[i].type == 9 )
     {
         fprintf(best, "\trhos     %.4lf\n", lens[i].pmass);
@@ -800,11 +862,23 @@ static void writePotentiel(FILE *best, long int i, int flag)
         fprintf(best, "\tend\n");
         return;
     }
+    if (lens[i].type == 17 )
+    {
+        fprintf(best, "\tdplxmap  %d %s\n", lens[i].dplxmap->format,lens[i].dplxmap->pixfile);
+        fprintf(best, "\tdplymap  %d %s\n", lens[i].dplymap->format,lens[i].dplymap->pixfile);
+        fprintf(best, "\tkappamap  %d %s\n", lens[i].kappamap->format,lens[i].kappamap->pixfile);
+        fprintf(best, "\tgamma1map  %d %s\n", lens[i].gamma1map->format,lens[i].gamma1map->pixfile);
+        fprintf(best, "\tgamma2map  %d %s\n", lens[i].gamma2map->format,lens[i].gamma2map->pixfile);
+        fprintf(best, "\tpotenmap  %d %s\n", lens[i].potenmap->format,lens[i].potenmap->pixfile);
+        fprintf(best, "\tz_lens     %.4lf\n", lens[i].z);
+        fprintf(best, "\tend\n");
+        return;
+    }
     fprintf(best, "\tx_centre     %.6lf\n", lens[i].C.x);
     fprintf(best, "\ty_centre     %.6lf\n", lens[i].C.y);
-    if (lens[i].type != 0 && lens[i].type != 2 && lens[i].type != 7)
+    if (lens[i].type != 0 && lens[i].type != 2 && lens[i].type != 7 && lens[i].type != 814)
     {
-        fprintf(best, "\tellipticite     %.6lf\n", lens[i].emass);
+        fprintf(best, "\tellipticity     %.6lf\n", lens[i].emass);
         if ( lens[i].type == 121 )
         {
 		    fprintf(best,"\ttheta       %.6lf\n",lens[i].theta*RTD);  // triaxial NFW
@@ -833,6 +907,7 @@ static void writePotentiel(FILE *best, long int i, int flag)
         fprintf(best, "\trc_slope        %.6lf\n", lens[i].rcslope);/////////////////
         fprintf(best, "\talpha           %.6lf\n", lens[i].alpha);
     }
+
     else if( lens[i].type == 16 )
     {
         if( flag & 1 ) fprintf(best, "\tscale_radius      %.6lf\n", lens[i].rc);
@@ -841,7 +916,7 @@ static void writePotentiel(FILE *best, long int i, int flag)
     }
     else
     {
-        if (lens[i].type != 0 && abs(lens[i].type) != 1 && lens[i].type != 7)
+        if (lens[i].type != 0 && abs(lens[i].type) != 1 && lens[i].type != 7 && lens[i].type != 814)
         {
             if( flag & 1 ) fprintf(best, "\tcore_radius         %.6lf\n", lens[i].rc);
             if( flag & 2 ) fprintf(best, "\tcore_radius_kpc     %.6lf\n", lens[i].rckpc );
@@ -858,8 +933,11 @@ static void writePotentiel(FILE *best, long int i, int flag)
             fprintf(best, "\tsigma_e     %.6le\n", lens[i].sigma);
             fprintf(best, "\tn     %.6lf\n", lens[i].alpha);
         }
-        else
-            fprintf(best, "\tv_disp     %.6lf\n", lens[i].sigma);
+        else{
+            if ( lens[i].type != 814){
+                fprintf(best, "\tv_disp     %.6lf\n", lens[i].sigma);
+            }
+        }
 
         if ( lens[i].type == 3 || lens[i].type == 6 || lens[i].type == 84 ||
              lens[i].type == 87 || lens[i].type == 88 )
@@ -871,8 +949,29 @@ static void writePotentiel(FILE *best, long int i, int flag)
             fprintf(best, "\trc_slope     %.6lf\n", lens[i].rcslope);
         }
 
-        if (lens[i].type == 7)
+        if (lens[i].type == 7){
             fprintf(best, "\tmasse     %.6lf\n", lens[i].masse);
+        }
+        
+        //Benjamin Beauchesne - Bspline modification
+        if ( lens[i].type == 814 || lens[i].type == 815 )
+        {
+            fprintf(best, "\tx_centre_pert     %.6lf\n", lens[i].Bspline.x_c);
+            fprintf(best, "\ty_centre_pert     %.6lf\n", lens[i].Bspline.y_c);
+            fprintf(best, "\tdegx     %d\n", lens[i].Bspline.degx);
+            fprintf(best, "\tdegy     %d\n", lens[i].Bspline.degy);
+            fprintf(best, "\tn_coeff     %d\n", lens[i].Bspline.n);
+	    fprintf(best, "\tangle_pos       %.6lf\n",lens[i].theta*RTD);
+            fprintf(best, "\tsize_patch     %.6lf\n", lens[i].Bspline.size_sq*(lens[i].Bspline.n+1.0));
+            fprintf(best, "\tfile_coeff     Coeff_best_%ld.dat\n",i+1);
+            
+            char name[30];
+            snprintf(name, 30,"rm Coeff_best_%ld.dat",(i+1)); 
+            system(name); 
+            writeBspline_coeff(lens[i].Bspline,i);
+            
+        }
+        
     }
 
     if (lens[i].mag != 0)
@@ -896,6 +995,21 @@ static void writeLimit(FILE *best, long int i)
     if ( block[i][CY] != 0 )
         fprintf(best, "\ty_centre     %d %.6lf %.6lf %.6lf\n", block[i][CY],
                 lmin[i].C.y, lmax[i].C.y, prec[i].C.y);
+    //Benjamin Beauchesne - Bspline modification
+    
+    if ( block[i][CX_PERT] != 0 )
+        fprintf(best, "\tx_centre_pert     %d %.6lf %.6lf %.6lf\n", block[i][CX_PERT],
+                lmin[i].Bspline.x_c, lmax[i].Bspline.x_c, prec[i].Bspline.x_c);
+     if ( block[i][SIZE_LATT] != 0 )
+        fprintf(best, "\tsize_patch     %d %.6lf %.6lf %.6lf\n", block[i][SIZE_LATT],
+                lmin[i].Bspline.size_sq*(lens[i].Bspline.n+1.0), lmax[i].Bspline.size_sq*(lens[i].Bspline.n+1.0), prec[i].Bspline.size_sq);
+    if ( block[i][CY_PERT] != 0 )
+        fprintf(best, "\ty_centre_pert     %d %.6lf %.6lf %.6lf\n", block[i][CY_PERT],
+                lmin[i].Bspline.y_c, lmax[i].Bspline.y_c, prec[i].Bspline.y_c);
+    if ( block[i][CY_PERT+1] != 0 )
+
+        fprintf(best, "\tBs_coeff     %d %.6lf %.6lf %.6lf\n", block[i][CY_PERT+1],
+                lmin[i].Bspline.Pz[0][1], lmax[i].Bspline.Pz[0][1], prec[i].Bspline.Pz[0][1]);
     if (lens[i].type != 0 && lens[i].type != 2 && lens[i].type != 7)
     {
         if ( block[i][EPOT] != 0 )
@@ -904,7 +1018,7 @@ static void writeLimit(FILE *best, long int i)
         {
             if(lens[i].type!=14)
             {
-               fprintf(best, "\tellipticite     %d %.6lf %.6lf %.6lf\n", block[i][EMASS],
+               fprintf(best, "\tellipticity     %d %.6lf %.6lf %.6lf\n", block[i][EMASS],
                     lmin[i].emass, lmax[i].emass, prec[i].epot);
             }
             else
@@ -1018,3 +1132,48 @@ static void writeLimit(FILE *best, long int i)
 
     fprintf(best, "\tend\n");
 }
+
+
+//Benjamin Beauchesne - Bspline modification
+static void writeBspline_coeff(struct Bspline2D Bspline,int i){
+
+    FILE *file;
+    int k,l;
+    char name[30];
+    
+
+
+    snprintf(name,30,"Coeff_best_%d.dat",(i+1));
+    file=fopen(name,"a");
+    
+    for (k=0;k<Bspline.n;k++){
+        for (l=0;l<Bspline.n;l++){
+            if ((k==Bspline.n-1) && (l==Bspline.n-1)) {
+                fprintf(file,"%f",Bspline.Pz[l+Bspline.degx+1][k+Bspline.degy+1]);
+            }
+            else {
+                fprintf(file,"%f ",Bspline.Pz[l+Bspline.degx+1][k+Bspline.degy+1]);
+            }
+        }
+    }
+    fprintf(file,"\n");
+    fclose(file);
+
+}
+// Not use for now
+static void writeBspline_limit(struct Bspline2D Bspline){
+
+    FILE *file;
+    int k,l;
+
+    file=fopen("Coeff_limit.dat","a");
+    
+    for (k=0;k<Bspline.n;k++){
+        for (l=0;l<Bspline.n;l++){
+            fprintf(file,"%f ",Bspline.Pz[k+Bspline.degx+1][l+Bspline.degy+1]);
+        }
+    }
+    fprintf(file,"e\n");
+    fclose(file);
+
+}
