@@ -6,6 +6,7 @@
 #include "dimension.h"
 #include "structure.h"
 #include "fonction.h"
+#include<gsl/gsl_cdf.h> 
 
 /********************************************************/
 /*      fonction: recale            */
@@ -70,6 +71,7 @@ int rescaleParam(int id, int type, int ipx, double *pval)
         extern int cblock[NPAMAX];
         extern struct pot         lmin[], lmax[];
         extern struct g_cosmo         clmin, clmax;
+        int k,l;
 
         switch (ipx)
         {
@@ -164,6 +166,27 @@ int rescaleParam(int id, int type, int ipx, double *pval)
                 val = prior( cblock[WA], val,
                              clmin.wa, clmax.wa );
                 break;
+            //Benjamin Beauchesne - Bspline modification
+            case(CX_PERT):
+                val = prior( block[id][CX_PERT], val,
+                             lmin[id].Bspline.x_c, lmax[id].Bspline.x_c );
+                break;
+            case(SIZE_LATT):
+                val = prior( block[id][SIZE_LATT], val,
+                             lmin[id].Bspline.size_sq, lmax[id].Bspline.size_sq );
+                break;
+            case(CY_PERT):
+                val = prior( block[id][CY_PERT], val,
+                             lmin[id].Bspline.y_c, lmax[id].Bspline.y_c );
+                break;
+            default:
+                break;
+        };
+        if (ipx>CY_PERT){ 
+            k=(ipx-CY_PERT-1)/lmin[id].Bspline.n;
+            l=(ipx-CY_PERT-1)%lmin[id].Bspline.n; 
+            val = prior( block[id][ipx], val,
+                             lmin[id].Bspline.Pz[l][k], lmax[id].Bspline.Pz[l][k]);
         };
         //if ( valid && ipx != B0 )
         //    o_set_lens( id, ipx, val );
@@ -261,9 +284,6 @@ int rescaleParam(int id, int type, int ipx, double *pval)
             case(SA):
                 val = prior(sblock[id][SA], val, smin[id].E.a, smax[id].E.a);
                 break;
-            case(SB):
-                val = prior(sblock[id][SB], val, smin[id].E.b, smax[id].E.b);
-                break;
             case(SEPS):
                 val = prior(sblock[id][SEPS], val, smin[id].eps, smax[id].eps);
                 break;
@@ -276,6 +296,21 @@ int rescaleParam(int id, int type, int ipx, double *pval)
             case(SFLUX):
                 val = prior(sblock[id][SFLUX], val, smin[id].mag, smax[id].mag);
                 break;
+            case(SA2):
+                val = prior(sblock[id][SA2], val, smin[id].E2.a, smax[id].E2.a);
+                break;
+            case(SEPS2):
+                val = prior(sblock[id][SEPS2], val, smin[id].eps2, smax[id].eps2);
+                break;
+            case(STHETA2):
+                val = prior(sblock[id][STHETA2], val, smin[id].E2.theta, smax[id].E2.theta);
+                break;
+            case(SINDEX2):
+                val = prior(sblock[id][SINDEX2], val, smin[id].var2, smax[id].var2);
+                break;
+            case(SFLUX2):
+                val = prior(sblock[id][SFLUX2], val, smin[id].mag2, smax[id].mag2);
+                break;
         }
     }
 
@@ -361,13 +396,21 @@ static double uniformPrior(double val, double min, double max)
 double dierfc(double y);
 /* Uniform [0:1] -> Gaussian[mean=mu, variance=sigma**2]
  */
-static double gaussianPrior(double val, double mu, double sigma)
+/*static double gaussianPrior(double val, double mu, double sigma)
 {
     double  sqrtTwo = 1.414213562;
 
     return mu + sigma*sqrtTwo*dierfc(2. * (1. - val) );
+}*/
+
+//Benjamin Clément 23th June 2020 now GSL implementation
+
+double gaussianPrior(double val, double mu, double sigma)
+{
+    return mu + gsl_cdf_gaussian_Pinv(val,sigma);
 }
 
+
 /* Inverse of errror function in double precision
  */
 static const int n_ck = 60;
@@ -530,15 +573,23 @@ int rescaleCube_1Atom(double *cube, int npar)
         if( block[ilens][CX] == 4 && block[ilens][CX] == 4 )
         {
             double r = 0.5 * cube[ipar];
-            double theta = cube[ipar+1] * 2. * M_PI;
+            double theta = cube[ipar+1] * 2. * PI;
             cube[ipar] = 0.5 + r * cos(theta);
             cube[ipar+1] = 0.5 + r * sin(theta);
         }
 		
 		// normal scaling for all the parameters
-        for ( ipx = CX; ipx <= PMASS; ipx++ )
-            if ( block[ilens][ipx] != 0 )
+        for ( ipx = CX; ipx <= PMASS; ipx++ ){
+            if ( block[ilens][ipx] != 0 ){
                 valid *= rescaleParam(ilens, POTENTIAL, ipx, &cube[ipar++]);
+            }
+        }
+        //Benjamin Beauchesne - Bspline modification
+        for ( ipx = CX_PERT; ipx < NPAMAX; ipx++ ){
+            if ( block[ilens][ipx] != 0 ){
+                valid *= rescaleParam(ilens, POTENTIAL, ipx, &cube[ipar++]);
+            }
+        }
     }
 
 // Rescale the clumps of the multiscale grid
@@ -548,7 +599,7 @@ int rescaleCube_1Atom(double *cube, int npar)
 // Rescale the source parameters
     if ( M.iclean == 2 )
         for ( k = 0; k < S.ns; k++ )
-            for ( ipx = SCX; ipx <= SFLUX; ipx++ )
+            for ( ipx = SCX; ipx <= SFLUX2; ipx++ )
                 if ( sblock[k][ipx] != 0 )
                     valid *= rescaleParam(k, SOURCES, ipx, &cube[ipar++]);
 
@@ -609,13 +660,6 @@ int rescaleCube_1Atom(double *cube, int npar)
     for ( ilens = 0; ilens < G.no_lens; ilens++ )
         if ( lens[ilens].b0 != 0. && lens[ilens].rc > lens[ilens].rcut )   valid = 0;
 
-    if ( M.iclean == 2 )
-    {
-        extern struct galaxie source[NFMAX];
-        for ( k = 0; k < S.ns; k++ )
-            if( source[k].E.a < source[k].E.b )   valid = 0;
-    }
-
     return valid;
 }
 
@@ -638,9 +682,17 @@ int rescaleCube_NAtoms(double **cubes, int natoms, int npar)
     for ( ilens = 0; ilens < G.no_lens; ilens++ )
     {
         ipar = 0;
-        for ( ipx = CX; ipx <= PMASS; ipx++ )
-            if ( block[ilens][ipx] != 0 )
+        for ( ipx = CX; ipx <= PMASS; ipx++ ){
+            if ( block[ilens][ipx] != 0 ){
                 valid *= rescaleParam(ilens, POTENTIAL, ipx, &cubes[ilens][ipar++]);
+            }
+        }
+        //Benjamin Beauchesne - Bspline modification
+        for ( ipx = CX_PERT; ipx < NPAMAX; ipx++ ){
+            if ( block[ilens][ipx] != 0 ){
+                valid *= rescaleParam(ilens, POTENTIAL, ipx, &cubes[ilens][ipar++]);
+            }
+        }
     }
  
     // no need to go further??
