@@ -10,6 +10,7 @@
 #include "bayesys3.h"
 #include "userstr.h"
 #include "lt.h"
+#include "o_run_bayes.h"
 
 /****************************************************************/
 /*      nom:        o_run_bayes         */
@@ -22,17 +23,9 @@
  */
 typedef void (*sighandler_t)(int);
 static void signalReset(int);
-int optInterrupt;   // Global variable read in bayesapp.c/UserMonitor()
-double  *tmpCube;      // temporary Cube to be used in UserBuild()
-double  **zbitstmp1;
-int    *ibitstmp;
 
-// Mutex functions implemented in bayesapp.c
-void init_mutex();
-void destroy_mutex();
-
-// Variables to store the likelihood and restart
-FILE *fh_restart;
+UserCommonStr *userCommon_init(int nDim);
+void userCommon_free(UserCommonStr *userCommon);
 
 double  o_run_bayes()
 {
@@ -43,13 +36,14 @@ double  o_run_bayes()
 // Addressing
     CommonStr     Common[1];
     ObjectStr     *Objects;
-    UserCommonStr UserCommon[1];
+    UserCommonStr *UserCommon;
     int           nDim;
     long int      i, j;
     FILE *burnin;
 
     // Get the number of free parameters, and write header of bayes.dat
     nDim = bayesHeader();
+    
 
 #ifdef DEBUG
     burnin = fopen( "burnin.dbg.dat", "w" );
@@ -332,29 +326,17 @@ double  o_run_bayes()
  
     }
     // Initialize some statistics
+    UserCommon = userCommon_init(nDim);
     Common->UserCommon = (void*)UserCommon;
-    UserCommon->Nsample = 0;
-    UserCommon->atoms   = 0.0;
-    UserCommon->Nchi2 = 0;
-    UserCommon->sum = 0.;
-    UserCommon->Ndim = nDim;
-
-    NPRINTF(stderr,"UserCommon->Ndim = %d\n",UserCommon->Ndim);
     tmpCube = (double *)malloc((unsigned int) nDim * sizeof(double));
-    UserCommon->err = (double *)malloc((unsigned int) nDim * sizeof(double));
-    UserCommon->avg = (double *)malloc((unsigned int) nDim * sizeof(double));
-    for ( i = 0; i < nDim; i++ )
-    {
-        UserCommon->err[i] = 0.;
-        UserCommon->avg[i] = 0.;
-    }
+    NPRINTF(stderr,"UserCommon->Ndim = %ld\n", UserCommon->Ndim);
 
     // Handle CTRL-C to interrupt the optimisation but not lenstool
     signal(SIGINT, signalReset);
     optInterrupt = 0;
     // Initialize mutex for nchi2 counts
     init_mutex();
-
+ 
     //Run bayesys
     int code = BayeSys3(Common, Objects);
     printf("\nBayeSys3 return code %d\n", code);
@@ -399,8 +381,7 @@ double  o_run_bayes()
         free( Objects->Mock );
     }
     free( Objects );
-    free( UserCommon->err );
-    free( UserCommon->avg );
+    userCommon_free(UserCommon);
     free( tmpCube );
 
     return( Common->Evidence );
@@ -446,14 +427,23 @@ int bayesHeader()
 
 
 // Number of lens parameters in the atom
-    for ( i = 0; i < G.no_lens; i++ )
-        for ( ipx = CX; ipx <= PMASS; ipx++ )
+    for ( i = 0; i < G.no_lens; i++ ){ 
+        for ( ipx = CX; ipx <= PMASS; ipx++ ){
             if ( block[i][ipx] != 0 )
             { 
                 nDim++;
                 fprintf( bayes, "#%s : %s\n", lens[i].n, getParName(ipx, name, lens[i].type) );
-            }
-
+            };
+        };
+        //Benjamin Beauchesne - Bspline modification
+        for ( ipx = CX_PERT; ipx < NPAMAX; ipx++ ){
+            if ( block[i][ipx] != 0 )
+            {
+                nDim++;
+                fprintf( bayes, "#%s : %s\n", lens[i].n, getParName(ipx, name, lens[i].type) );
+            };
+        };
+    };
     // multiscale grid clumps : code enters the loop below for the test case
     for ( i = G.nmsgrid; i < G.nlens; i++ )
     { 
@@ -486,7 +476,7 @@ int bayesHeader()
         extern struct g_source  S;
 
         for ( i = 0; i < S.ns; i++ )
-            for ( ipx = SCX; ipx <= SFLUX; ipx++ )
+            for ( ipx = SCX; ipx <= SFLUX2; ipx++ )
                 if ( sblock[i][ipx] != 0 )
                 {
                     nDim++;
@@ -618,3 +608,36 @@ static void signalReset(int param)
     optInterrupt = 1;
     fprintf( stderr, "INFO: Optimisation interrupted by CTRL-C\r");
 }
+
+UserCommonStr *userCommon_init(int nDim)
+{
+    UserCommonStr *userCommon;
+    userCommon = (UserCommonStr *)malloc(sizeof(UserCommonStr));
+
+    userCommon->avg = (double *)malloc(nDim*sizeof(double));
+    userCommon->err = (double *)malloc(nDim*sizeof(double));
+
+    userCommon->Nsample = 0;
+    userCommon->atoms   = 0.0;
+    userCommon->Nchi2 = 0;
+    userCommon->sum = 0.;
+    userCommon->Ndim = nDim;
+
+    int i;
+
+    for ( i = 0; i < nDim; i++ )
+    {
+        userCommon->err[i] = 0.;
+        userCommon->avg[i] = 0.;
+    }
+
+    
+    return userCommon;
+}
+
+void userCommon_free(UserCommonStr *userCommon)
+{
+    free(userCommon->avg);
+    free(userCommon->err);
+    free(userCommon);
+}
