@@ -74,9 +74,15 @@ double  **readBayesModels(int *nParam, long int *nVal)
     // Get G.no_lens from bayes.dat
     // Reset G and block global variables
     G.no_lens = 0;
-    for ( ilens = 0; ilens < G.nlens; ilens++)
-        for ( ipx = CX; ipx <= PMASS; ipx ++)
+    for ( ilens = 0; ilens < G.nlens; ilens++){
+        for ( ipx = CX; ipx <= PMASS; ipx ++){
             block[ilens][ipx] = 0;
+        }
+        //Benjamin Beauchesne - Bspline modification
+        for ( ipx = CX_PERT; ipx < NPAMAX; ipx ++){ 
+            block[ilens][ipx] = 0;
+        }
+    }
 
     // Source parameters
 //    for ( i = 0; i < S.ns; i++ )
@@ -224,9 +230,17 @@ void    setBayesModel( long int iVal, long int nVal, double **array)
 
     for ( ilens = 0; ilens < G.no_lens; ilens++ )
     {
-        for ( ipx = CX; ipx <= PMASS; ipx++ )
-            if ( block[ilens][ipx] != 0 )
+        for ( ipx = CX; ipx <= PMASS; ipx++ ){
+            if ( block[ilens][ipx] != 0 ){
                 o_set_lens( ilens, ipx, method(nVal, iParam++, array) );
+            }
+        }
+        //Benjamin Beauchesne - Bspline modification
+        for ( ipx = CX_PERT; ipx < NPAMAX; ipx++ ){
+            if ( block[ilens][ipx] != 0 ){
+                o_set_lens( ilens, ipx, method(nVal, iParam++, array) );
+            }
+        }
 
         // TODO in o_set_lens() assign sigma directly to lens.sigma rather than lens.b0
         if ( block[ilens][B0] != 0 )
@@ -290,12 +304,20 @@ void    setBayesModel( long int iVal, long int nVal, double **array)
             if( pch != NULL )
                 source[i].C.x = source[i].C.y = 0.;
 
-            for( ipx = SCX ; ipx <= SFLUX; ipx++ )
+            for( ipx = SCX ; ipx <= SFLUX2; ipx++ )
                 if( sblock[i][ipx] != 0 )
                     o_set_source( &source[i], ipx, method(nVal, iParam++, array));
 
-            if( sblock[i][SEPS] != 0 )
+    	    if((sblock[i][SA] !=0) & (source[i].E.a<0))
+		    source[i].E.a=-source[i].E.a;
+    	    if((sblock[i][SA2] !=0) & (source[i].E2.a<0))
+		    source[i].E2.a=-source[i].E2.a;
+
+            if((sblock[i][SEPS] != 0 )|| (sblock[i][SA] !=0))
                 source[i].E.b = source[i].E.a * (1. - source[i].eps) / (1. + source[i].eps);
+            if((sblock[i][SEPS2] != 0 )|| (sblock[i][SA2] !=0))
+                source[i].E2.b = source[i].E2.a * (1. - source[i].eps2) / (1. + source[i].eps2);
+
        }
     }
 
@@ -600,9 +622,9 @@ static double ** readBayesASCII(int *nParam, long int *nVal)
 
 // Read the header of bayes.fits and call analyseHeader()
 // to process each line
-static double ** readBayesFITS(int *nParam, long int *nVal)
-{
-}
+//static double ** readBayesFITS(int *nParam, long int *nVal)
+//{
+//}
 
 /* Analyse a line of the bayes.dat header and eventually
  * In the case of reading a best.par file, reset the
@@ -655,8 +677,9 @@ static int analyseHeader( char *line )
         {
             // ... and in Lenstool... <pch2> in Lenstool
             pch2 = lens[i].n;
-            if ( pch2[0] == 'O' )
+            if ( pch2[0] == 'O' ){
                 pch2++;
+            };
 
             if ( !strcmp( pch, pch2 ) )
             {
@@ -666,12 +689,26 @@ static int analyseHeader( char *line )
                 for ( ipx = CX; ipx <= PMASS; ipx++ )
                 {
                     pch2 = strtok( getParName(ipx, name, lens[i].type), " ");   // remove the unit if any
-                    if ( !strcmp( pch, pch2) && block[i][ipx] == 0  )
+                    if ( !strcmp( pch, pch2) && block[i][ipx] == 0  ){
                         block[i][ipx] = 1;
+                    };
                 }
                 // Prevent the case when the clump is already set as optimised
-                for ( ipx = CX; ipx <= PMASS; ipx++ )
+                for ( ipx = CX; ipx <= PMASS; ipx++ ){
                     updated += block[i][ipx];
+                };
+                //Benjamin Beauchesne - Bspline modification
+                for ( ipx = CX_PERT; ipx < NPAMAX; ipx++ )
+                {
+                    pch2 = strtok( getParName(ipx, name, lens[i].type), " ");   // remove the unit if any
+                    if ( !strcmp( pch, pch2) && block[i][ipx] == 0  ){
+                        block[i][ipx] = 1;
+                    }
+                }
+                // Prevent the case when the clump is already set as optimised
+                for ( ipx = CX_PERT; ipx < NPAMAX; ipx++ ){
+                    updated += block[i][ipx]; 
+                }
 
                 // Increment G.no_lens only once per clump
                 if ( updated == 1 )
@@ -824,14 +861,23 @@ static void convertArray(long int nVal, double **array )
         // ie same behavior as chi2
 
         // Optimized clumps
-        for ( i = 0; i < G.no_lens; i++ )
-            for ( ipx = CX; ipx <= PMASS; ipx++ )
+        for ( i = 0; i < G.no_lens; i++ ){
+            for ( ipx = CX; ipx <= PMASS; ipx++ ){
                 if ( block[i][ipx] != 0 )
                 {
                     array[nParam][iVal] = bayes2lt( i, ipx, array[nParam][iVal] );
                     nParam++;
                 }
-        
+            }
+            //Benjamin Beauchesne - Bspline modification
+            for ( ipx = CX_PERT; ipx < NPAMAX; ipx++ ){
+                if ( block[i][ipx] != 0 )
+                {
+                    array[nParam][iVal] = bayes2lt( i, ipx, array[nParam][iVal] );
+                    nParam++;
+                }
+             }
+        }
         // Grid clumps
         for ( i = G.nmsgrid; i < G.nlens; i++ )
             nParam++;
@@ -842,12 +888,13 @@ static void convertArray(long int nVal, double **array )
         // Sourcelimit parameters
         if( M.iclean == 2 )
             for ( i = 0; i < S.ns; i++ )
-                for( ipx = SCX; ipx <= SFLUX; ipx ++ )
+                for( ipx = SCX; ipx <= SFLUX2; ipx ++ )
                     if( sblock[i][ipx] != 0 )
                     {
                         if( ipx == STHETA )
                             array[nParam][iVal] = array[nParam][iVal] * DTR;
-    
+                        if( ipx == STHETA2 )
+                            array[nParam][iVal] = array[nParam][iVal] * DTR;
                         nParam ++;
                     }
     }
