@@ -94,9 +94,10 @@ void    set_lens()
             lens[i].theta = 0.;
             lens[i].type++;
         }
+        //Benjamin Beauchesne - Bspline modification
         else if ( lens[i].type ==  8  ||
                   lens[i].type == -2  ||
-                  ( lens[i].type > 80 && lens[i].type < 90 ) )
+                  ( lens[i].type > 80 && lens[i].type < 90 ))
         {
             if ( lens[i].emass == 0. && lens[i].epot != 0. )
                 // emass is (a2-b2)/(a2+b2)
@@ -107,6 +108,58 @@ void    set_lens()
                 // epot is (a-b)/(a+b)
                 lens[i].epot = (1. - sqrt(1 - lens[i].emass * lens[i].emass)) / lens[i].emass;
         }
+        else if (lens[i].type==814 )
+        {
+            if ( lens[i].emass == 0. && lens[i].epot != 0. )
+                // emass is (a2-b2)/(a2+b2)
+                lens[i].emass = 2.*lens[i].epot / (1. + lens[i].epot * lens[i].epot);
+            else if ( lens[i].emass == 0. && lens[i].epot == 0. )
+                lens[i].epot = 0.00001;
+            else
+                // epot is (a-b)/(a+b)
+                lens[i].epot = (1. - sqrt(1 - lens[i].emass * lens[i].emass)) / lens[i].emass;
+            lens[i].Bspline.ny=lens[i].Bspline.n+2*(lens[i].Bspline.degx+1);
+            lens[i].Bspline.nx=lens[i].Bspline.n+2*(lens[i].Bspline.degy+1);
+            if (lens[i].Bspline.lim !=0.0) lens[i].Bspline.size_sq=lens[i].Bspline.lim/(lens[i].Bspline.n+1.0);
+            //lens[i].Bspline.x_c+=lens[i].C.x;
+            //lens[i].Bspline.y_c+=lens[i].C.y; // define relative center// not needed
+            knotx_init(&lens[i].Bspline);
+            knoty_init(&lens[i].Bspline);
+            lens[i].Bspline.Pz=init_ctrl_pts(&lens[i].Bspline); 
+            int k,l;
+            // For now it is a necessity to put perturbative potential in first in the parfile
+            // due to the next function
+            get_coeff_from_file(lens[i].Bspline.coeff_file,&lens[i].Bspline,i);
+
+            init_derivate_Bspline(&lens[i].Bspline);
+             
+        }
+        else if (lens[i].type==815)
+        {
+            if ( lens[i].emass == 0. && lens[i].epot != 0. )
+                // emass is (a2-b2)/(a2+b2)
+                lens[i].emass = 2.*lens[i].epot / (1. + lens[i].epot * lens[i].epot);
+            else if ( lens[i].emass == 0. && lens[i].epot == 0. )
+                lens[i].epot = 0.00001;
+            else
+                // epot is (a-b)/(a+b)
+                lens[i].epot = (1. - sqrt(1 - lens[i].emass * lens[i].emass)) / lens[i].emass;
+            if (lens[i].Bspline.lim !=0.0) lens[i].Bspline.size_sq=lens[i].Bspline.lim/(lens[i].Bspline.n+1.0);
+            lens[i].Bspline.ny=lens[i].Bspline.n+2*(lens[i].Bspline.degx+1);
+            lens[i].Bspline.nx=lens[i].Bspline.n+2*(lens[i].Bspline.degy+1);
+            //lens[i].Bspline.x_c+=lens[i].C.x;
+            //lens[i].Bspline.y_c+=lens[i].C.y; // define relative center// not needed
+            knotx_init(&lens[i].Bspline);
+            knoty_init(&lens[i].Bspline);
+            lens[i].Bspline.Pz=init_ctrl_pts(&lens[i].Bspline);
+            int k,l;
+            // For now it is a necessity to put perturbative potential in first in the parfile
+            // due to the next function
+            get_coeff_from_file(lens[i].Bspline.coeff_file,&lens[i].Bspline,i);
+
+            inite_power_spectrum(&lens[i].Bspline,1.0,128);
+             
+        }
         else
         {
             if ( lens[i].emass == 0. && lens[i].epot != 0. )
@@ -261,6 +314,21 @@ void    set_lens()
 		lmax[i].b0=lmax[i].pmass*1e-12/d1;
 		prec[i].b0=prec[i].pmass*1e-12/d1;
 	}
+        //Benjamin Beauchesne - Bspline modification 
+        // Each coefficient have same limit and are strored in the array of
+        // control points.
+        else if( lens[i].type==814 || lens[i].type==815){
+            lmin[i].Bspline.n=prec[i].Bspline.n=lmax[i].Bspline.n=lens[i].Bspline.n;
+            lmin[i].Bspline.degx=prec[i].Bspline.degx=lmax[i].Bspline.degx=lens[i].Bspline.degx;
+            lmin[i].Bspline.degy=prec[i].Bspline.degy=lmax[i].Bspline.degy=lens[i].Bspline.degy;
+            //lmin[i].Bspline.nx=prec[i].Bspline.nx=lmax[i].Bspline.nx=lens[i].Bspline.nx;
+            //lmin[i].Bspline.ny=prec[i].Bspline.ny=lmax[i].Bspline.ny=lens[i].Bspline.ny;
+
+            lmin[i].Bspline.Pz=init_ctrl_pts_lim(&lmin[i].Bspline,&lens[i].Bspline,block[i],fabs(lmin[i].Bspline.lim),0,lmin[i].Bspline.x_c,lmin[i].Bspline.ny);
+            lmax[i].Bspline.Pz=init_ctrl_pts_lim(&lmax[i].Bspline,&lens[i].Bspline,block[i],fabs(lmax[i].Bspline.lim),1,lmin[i].Bspline.x_c,lmin[i].Bspline.ny);
+            prec[i].Bspline.Pz=init_ctrl_pts_prec(&prec[i].Bspline,prec[i].Bspline.lim);
+            //printf("%f %f %f \n",lmax[i].Bspline.lim,lmin[i].Bspline.Pz[0][0],prec[i].Bspline.Pz[0][0]);
+        }
         else
         {
             lmin[i].b0 = 6.*pia_c2 * lmin[i].sigma * lmin[i].sigma;
@@ -294,7 +362,7 @@ void set_dynamics(long int i)
     extern struct g_grille G;
     extern struct pot      lens[];
     extern struct g_cosmo  C;
-    
+    extern int block[][NPAMAX]; 
     register int ii, jj;
     double test;
 
@@ -466,15 +534,49 @@ void set_dynamics(long int i)
             /*          NPRINTF(stderr,"Clump %d: trunc. PIEMD Kovner:",i);
                         NPRINTF(OUT,"Clump %d: PIEMD Kovner, truncated\n",i);
                         NPRINTF(OUT," Total mass: %lf(cor) %lf (10^12 M_sol)\n",
-                            4*M_PI/3*M_PI/GG*(ilens->sigma/1000)*(ilens->sigma/1000)*
+                            4*PI/3*PI/GG*(ilens->sigma/1000)*(ilens->sigma/1000)*
                             ilens->rcut*(d0/C.h*distcosmo1(ilens->z)),
-                            1.5*M_PI/GG*(ilens->sigma/1000)*(ilens->sigma/1000)*
+                            1.5*PI/GG*(ilens->sigma/1000)*(ilens->sigma/1000)*
                         ilens->rcut*(d0/C.h*distcosmo1(ilens->z)) );
                         NPRINTF(OUT," rcut:%.2lf(\") %.2lf(kpc)\n",
                             ilens->rcut,ilens->rcut*(d0/C.h*distcosmo1(ilens->z)) );
             */
             ilens->b0 = 6.*pia_c2 * ilens->sigma * ilens->sigma;
             break;
+        case(814):
+            //Benjamin Beauchesne - Bspline modification
+            // initialize for PIEMD
+            ilens->b0 = 6.*pia_c2 * ilens->sigma * ilens->sigma;
+
+            // Initialise Bspline surface 
+            knotx_init(&ilens->Bspline);
+            knoty_init(&ilens->Bspline);
+            //int k;
+            //for (k=0;k<(ilens->Bspline.nx+ilens->Bspline.degx+1);k++) printf("knotx[%d]%f \n",k,ilens->Bspline.kx[k]);
+            
+            
+            
+            // initialise Derivate controls points
+            update_derivate_Bspline(&ilens->Bspline);
+            break;
+        case(815):
+            //Benjamin Beauchesne - Bspline modification
+            // initialize for PIEMD
+            ilens->b0 = 6.*pia_c2 * ilens->sigma * ilens->sigma;
+
+            // Initialise Bspline surface
+            knotx_init(&ilens->Bspline);
+            knoty_init(&ilens->Bspline);
+            //int k;
+            //for (k=0;k<(ilens->Bspline.nx+ilens->Bspline.degx+1);k++) printf("knotx[%d]%f \n",k,ilens->Bspline.kx[k]);
+            
+            
+            
+            //              
+            break;
+
+
+
 
         case(82):
             /*          NPRINTF(stderr,"Clump %d: PIEMD Kovner, shallow center\n",i);
